/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.consumers;

import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.agents.serviceproviders.innovation.Innovation;
import edu.ntnu.ecosim.basicfunctionality.Config;
import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.environment.Market;
import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import java.util.ArrayList;
import java.util.Random;

/**
 * This behaviour model is based on "Diffusion of innovation theory". 
 * The behaviour will reflect the "willingness" to adopt the new technology,  
 * based on the characteristics of each user type.
 * @author   Martin
 */
public class BehaviourModel {

	/** The Constant INNOVATOR. */
	private static final int INNOVATOR = 0; //2.5%

	/** The Constant EARLY_ADOPTER. */
	private static final int EARLY_ADOPTER = 1;  //13.5 %

	/** The Constant EARLY_MAJORITY. */
	private static final int EARLY_MAJORITY = 2; //34%

	/** The Constant LATE_MAJORITY. */
	private static final int LATE_MAJORITY = 3;  // 34%

	/** The Constant LAGGARD. */
	private static final int LAGGARD = 4;	// 16%

	/** The user type of the consumer having this behavior model. */
	private int usertype;

	/**
	 * The max friends this consumer may have.
	 */
	private int maxFriends;

	/**
	 * The influence this consumer has on other consumers
	 */
	private int influence;

	/**
	 * The memory of the consumer
	 */
	private Memory memory;

	/**
	 * The market the consumer operates in
	 */
	private Market market;

	/**
	 * The noise to be added to the perceived value of the providers
	 */
	private double valuenoise;


	/**
	 * The noise to be added to the perceived number of other subscribers
	 */
	private boolean subscribernoise;

	/**
	 * The probability that the consumer searches for service providers in the market
	 */
	private int providerSearchProbability;

	/**
	 * A random number generator
	 */
	private Random rand;


	/**
	 * This is the constructor of the Behaviour Model The category of the
	 * consumer is selected based on the technology acceptance curve.
	 * 
	 * @param memory the memory of the consumer
	 * @param m the market the consumer operates within
	 */
	protected BehaviourModel(final Memory memory, final Market m){
		rand = new Random();
		this.memory = memory;
		market = m;
		final double chooser = Math.random()*100;
		if(chooser<=2.5){
			usertype = INNOVATOR;
		}
		else if((2.5<chooser)&&(chooser<=16.0)){
			usertype = EARLY_ADOPTER;
		}
		else if((16.0<chooser)&&(chooser<=50.0)){
			usertype = EARLY_MAJORITY;
		}
		else if((50<chooser)&&(chooser<=84.0)){
			usertype = LATE_MAJORITY;
		}
		else{
			usertype = LAGGARD;
		}
		maxFriends = calculateMaxFriends();
		influence = getInfluenceFromConfig();
		try{
			valuenoise = Double.parseDouble(Config.getInstance().getProperty("ecosim","Consumers.ValueNoiseFactor"));
		}
		catch(final PropertyNotFoundException e){
			valuenoise = 0.01;
		}
		try{
			subscribernoise = Boolean.parseBoolean(Config.getInstance().getProperty("ecosim","Consumers.SubscriberNoiseFactor"));
		}
		catch(final PropertyNotFoundException e){
			subscribernoise = true;
		}
		try{
			final int interval = Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.ProviderSearchInterval"));
			providerSearchProbability = 100/interval;
		}
		catch(final PropertyNotFoundException e){
			providerSearchProbability = 33;
		}

	}


	/**
	 * Method to calculate the innvations network effects for the specified provider
	 * @param sp The service provider
	 * @param valuation The valuation of a basic subscription
	 * @param nofFriends the number of friends this consumer has
	 * @return A double giving the innovation effects pr. friend using this provider
	 */
	protected double[] calculateInnovationsEffects(final ServiceProvider sp, final double valuation, final int nofFriends){
		if(usertype==INNOVATOR){
			searchForInnovations(sp);
		}
		else if(usertype==EARLY_ADOPTER){
			if(rand.nextBoolean()) {
				searchForInnovations(sp);
			}
		}
		double totalValueAdd=0;
		double qTot = 1;
		for(int i=0;i<sp.getNofInnovations();i++){
			final Innovation inno = sp.getInnovation(i);
			if(memory.knowsInnovation(inno.getUniqueID())){
				if(inno.getType()==Innovation.NETEFFADDING){
					qTot = qTot*(1-inno.getNetEffect());
				}
				else{
					totalValueAdd+=(valuation*inno.getValueAdd());
				}
			}
		}
		qTot = 1-qTot;
		final double c =(qTot*valuation)/(nofFriends*(1-qTot));
		final double[] toReturn = {c,totalValueAdd}; 
		return toReturn;
	}


	/**
	 * Method to calculate the maximum number of friends this consumer
	 * is "allowed" to have. The algorithm used is shown to produce a "skewed" distribution
	 * of friends.
	 * @return the max number of friends
	 */
	private int calculateMaxFriends(){
		int globalMax;
		int globalMin;
		try {
			globalMax = Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.MaxFriends"));
			globalMin = Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.MinFriends"));
		} catch (final NumberFormatException e) {
			Log.getInstance().writeToLog("Could not convert into integer: "+e.getMessage(), Log.ERROR);
			globalMax = 20;
			globalMin = 5;
		} catch (final PropertyNotFoundException e) {
			Log.getInstance().writeToLog("Could not find property, setting default values.. : "+e.getMessage(), Log.ERROR);
			globalMax = 20;
			globalMin = 5;
		}
		final int steps = (globalMax-globalMin)/5;
		switch(usertype){
		case INNOVATOR:	
			return nextGaussian(globalMin+(steps*4.5), (double)steps/2, globalMax, globalMin);

		case EARLY_ADOPTER:
			return nextGaussian(globalMin+(steps*3.5), steps, globalMax, globalMin);
		case EARLY_MAJORITY:
			return nextGaussian(globalMin+(steps*2.5), steps, globalMax, globalMin);
		case LATE_MAJORITY:
			return nextGaussian(globalMin+(steps*1.5), steps, globalMax, globalMin);
		case LAGGARD:
			return nextGaussian(globalMin+((double)steps*1), steps, globalMax, globalMin);
		default: 
			return rand.nextInt(globalMax)+1;
		}
	}


	/**
	 * Method to calculate the valuation of a basic subscription 
	 * (the intrisic value of a subscription)
	 * @return a double giving the value
	 */
	protected double calculateValuation(){
		final ArrayList<String> known  = memory.getKnownProviders();
		if(known.size()>0){
			double value = 0;
			int divider = 0;
			for(int i=0;i<market.getNofProviders();i++){
				final ServiceProvider sp = market.getServiceProvider(i);
				if(known.contains(sp.getUniqueID())){
					value+=sp.getSubscriptionFee();
					divider++;
				}
			}
			final double value2 = value/divider;
			double diff = 0;
			if(usertype==INNOVATOR){
				diff = (value2*(Math.random()*valuenoise));
			}
			else if(usertype==EARLY_ADOPTER){
				diff = (value2*(Math.random()*(valuenoise*2)));
			}
			else if(usertype==EARLY_MAJORITY){
				diff = (value2*(Math.random()*(valuenoise*4)));
			}
			else if(usertype==LATE_MAJORITY){
				diff = (value2*(Math.random()*(valuenoise*6)));
			}
			else{
				diff = (value2*(Math.random()*(valuenoise*8)));
			}

			final java.util.Random randgen = new java.util.Random();
			final boolean add = randgen.nextBoolean();
			if(add){
				//Less chance of overvaluation than undervaluation
				diff = diff/2;
				return value2+diff;
			}
			else{
				return value2-diff;
			}
		}
		else{
			return 0;
		}
	}


	/**
	 * A method to get the binding fee of  a given service provider.
	 * 
	 * @return the binding fee of the provider
	 */
	protected double getBindingFee(final ServiceProvider sp){
		return sp.getBindingFee();
	}


	/**
	 * Method to get the influence this consumer has on others
	 * @return      the influence
	 */
	protected int getInfluence(){
		return influence;
	}



	/**
	 * Method used to get the correct influence from the config file
	 * @return the influence of this consumer
	 */
	private int getInfluenceFromConfig(){
		try{
			switch(usertype){
			case INNOVATOR: return Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.InnovatorInfluence"));
			case EARLY_ADOPTER: return Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.EarlyAdopterInfluence"));
			case EARLY_MAJORITY: return Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.EarlyMajorityInfluence"));
			case LATE_MAJORITY: return Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.LateMajorityInfluence"));
			case LAGGARD: return Integer.parseInt(Config.getInstance().getProperty("ecosim","Consumers.LaggardInfluence"));
			default: return 25;
			}
		}
		catch(final PropertyNotFoundException e){
			Log.getInstance().writeToLog("Influence levels not found in configuration. "+e.getMessage(), Log.ERROR);
			return 25;
		}
	}


	/**
	 * Method to get the maximum number of friends this consumer may have.
	 * @return         the max number of friends
	 */
	protected int getMaxFriends(){
		return maxFriends;
	}



	/**
	 * This method adds noise to the number of subscribers as percieved by the
	 * consumers. Laggards have less correct information than innovators. At most
	 * there may be 100% noise, which means that laggards may perceive a company
	 * with many subscribers to have none
	 * 
	 * @param sp
	 *            The service provider to get information from
	 * @param isCurrent
	 *            Whether or not this is the current provider
	 * 
	 * @return The assumed number of subscribers
	 */
	protected int getNofSubscribers(final ServiceProvider sp, final boolean isCurrent){
		int diff = 0;
		final int real = sp.getNofSubscribers();
		if(subscribernoise){
			if(real!=0){
				if(usertype==INNOVATOR){
					diff = (int)(real*(Math.random()*0.1));
				}
				else if(usertype==EARLY_ADOPTER){
					diff = (int)(real*(Math.random()*0.2));
				}
				else if(usertype==EARLY_MAJORITY){
					diff = (int)(real*(Math.random()*0.3));
				}
				else if(usertype==LATE_MAJORITY){
					diff = (int)(real*(Math.random()*0.4));
				}
				else{
					diff =  rand.nextInt(real);
				}
				if(isCurrent){
					diff = (int)(diff*0.5);
				}
			}
		}
		return real-diff;
	}




	/**
	 * A method to get the subscription fee of the service provider.
	 * 
	 * @param sp
	 *            the service provider
	 * @param isCurrent
	 *            Whether or not this is the current provider
	 * 
	 * @return the subscription fee
	 */
	protected double getSubscriptionFee(final ServiceProvider sp, final boolean isCurrent){
		return sp.getSubscriptionFee();
	}


	/**
	 * Method to get a number from a gaussian distribution. The method also specifies a
	 * maximum and minimum number, allowing the algorithm to operate within boundaries.
	 * 
	 * @param mean The mean of the normal (gaussian) distribution
	 * @param standardDeviation The standard deviation of the normal (gaussian) distribution
	 * @param max The maximum allowed return value
	 * @param min The minimum allowed return value
	 * @return The calculated value
	 */
	private int nextGaussian(final double mean, final double standardDeviation, final int max, final int min) {
		int ret = (int)(mean + rand.nextGaussian()*standardDeviation);
		if(ret>max||ret<min){
			//Giving it a second chance...
			ret = (int)(mean + rand.nextGaussian()*standardDeviation);
			if(ret>max||ret<min){
				ret = (int)(mean);
			}
		}
		return ret;
	}


	/**
	 * Method used when the consumer searches for new innovations by the specified provider
	 * @param sp The service provider to search for innovations from
	 */
	private void searchForInnovations(final ServiceProvider sp) {
		final int nofInno = sp.getNofInnovations();
		for(int i=0;i<nofInno;i++){
			final Innovation inno = sp.getInnovation(i);
			if(!memory.knowsInnovation(inno.getUniqueID())){
				memory.addInnovationKnowledge(inno.getUniqueID(), sp.getUniqueID());
			}
		}
	}


	/**
	 * Method used to search for new providers in the market. 
	 * The rate of the search is configurable through the configuration file. 
	 * 
	 */
	protected void searchForNewProviders() {
		if(rand.nextInt(100)<providerSearchProbability){
			if(usertype==INNOVATOR){
				final ArrayList<String> known = memory.getKnownProviders();
				if(known.size()==market.getNofProviders()){
					return;
				}
				else{
					for(int i=0;i<market.getNofProviders();i++){
						if(!known.contains(market.getServiceProvider(i).getUniqueID())){
							if(market.getServiceProvider(i).getNofInnovations()>0){
								memory.addKnowledge(market.getServiceProvider(i).getUniqueID(), 100);
								return;
							}
						}
					}
				}
			}
		}
	}



}
