/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.consumers;


import edu.ntnu.ecosim.agents.Agent;
import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.basicfunctionality.Config;
import edu.ntnu.ecosim.environment.Market;
import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

/**
 * This is the class that represent the consumer agents.
 * @author           Martin
 */
public class Consumer extends Agent{

	/** The service provider that this consumer is subscribing to. */
	private ServiceProvider sp;

	/** The behaviour model of the consumer. */
	private BehaviourModel bm;

	/** The received "return on investment" (ROI). */
	private double returnOnInvestment;

	/** The memory of this consumer. */
	private Memory memory;

	/**	Whether of not this consumer has changed provider this round */
	private boolean hasChangedProvider;
	
	/** Whether the network effects should be calculated by social connections (instead of global subscribers) */
	private boolean netEffectsAreLocal;

	/**
	 * The unique ID of the consumer */
	private int uniqueID;

	/** The list of friends for this consumer. */
	private FriendList friends;
	
	/** Whether information of service providers is global, or the information is sent through social networks */
	private boolean providerInfoIsGlobal;

	/**
	 * This is the basic constructor, creating a new user without service
	 * provider at a specific time.
	 * 
	 * @param ID  the unique ID of this consumer
	 * @param market the market the consumer operates within
	 */
	public Consumer(final int ID, final Market market){
		super(market);
		hasChangedProvider = false;
		uniqueID =  ID;
		memory = new Memory();
		returnOnInvestment = 0;
		bm = new BehaviourModel(memory, market);
		friends = new FriendList(bm.getMaxFriends());
		try {
			netEffectsAreLocal = Boolean.parseBoolean(Config.getInstance().getProperty("ecosim", "Consumers.NetEffIsLocal"));
		} catch (final PropertyNotFoundException e) {
			//Setting default to "true";
			netEffectsAreLocal = true;
		}
		try{
			providerInfoIsGlobal = Boolean.parseBoolean(Config.getInstance().getProperty("ecosim", "Consumers.ProviderInfoGlobal"));
		}
		catch(final PropertyNotFoundException e){
			//Setting default to false
			providerInfoIsGlobal = false;
		}
	}



	/**
	 * This constructor can be used when an initial base of users are to be
	 * simulated.
	 * 
	 * @param spro
	 *            the service provider this consumer has initially
	 * @param ID
	 *            the unique ID of this consumer
	 * @param market the market the consumer operates within
	 */
	public Consumer(final int ID, final ServiceProvider spro, final Market market){
		super(market);
		uniqueID = ID;
		memory = new Memory();
		memory.addKnowledge(spro.getUniqueID(), 100);
		bm = new BehaviourModel(memory, market);
		friends = new FriendList(bm.getMaxFriends());
		sp = spro;
		try {
			netEffectsAreLocal = Boolean.parseBoolean(Config.getInstance().getProperty("ecosim", "Consumers.NetEffIsLocal"));
		} catch (final PropertyNotFoundException e) {
			//Setting default to "true";
			netEffectsAreLocal = true;
		}
	}



	/**
	 * Method used to announce a change of provider to the friends in this
	 * consumers social network.
	 * 
	 * @param sproUniqueID
	 *            the unique ID of the new service provider
	 */
	private void announceProvider(final String sproUniqueID){
		for(int i=0;i<friends.size();i++){
			final int friend = friends.get(i);
			market.getConsumer(friend).receiveProviderInformation(sproUniqueID, bm.getInfluence(), memory.getInnovationsOfProvider(sproUniqueID));
		}
	}



	/**
	 * Method used to make this consumer become friend with another consumer.
	 * 
	 * @param friendID
	 *            the ID of the potential friend
	 * 
	 * @return true if it's room for this friend, else false
	 */
	public boolean becomeFriend(final int friendID){
		if(friends.size()<bm.getMaxFriends()){
			friends.add(friendID);
			return true;
		} else {
			return false;
		}
	}



	/**
	 * This method is used to calculate the benefits of an innovation provided
	 * by a service provider.
	 * 
	 * @param spro
	 *            the service provider
	 *            
	 * @return a number representing the benefit of selecting the company
	 */
	private double calculateBenefits(final ServiceProvider spro){
		boolean isCurrentProvider = false;
		if(sp!=null){
			if(sp.getUniqueID().equals(spro.getUniqueID())){
				isCurrentProvider = true;
			}
		}

		double p = bm.getBindingFee(spro);
		if(isCurrentProvider){
			p = 0;
		}
		int nofCons;
		int total;
		if(netEffectsAreLocal){
			nofCons = friends.size();
			total = nofFriendsUsingSP(spro.getUniqueID());
		}
		else{
			nofCons = market.getNofConsumers();
			total = bm.getNofSubscribers(spro, true);
		}
		final double iv = bm.calculateValuation();
		final double k = bm.getSubscriptionFee(spro, isCurrentProvider);
		final double[] innovationEffects = bm.calculateInnovationsEffects(spro, iv, nofCons);
		final double c = innovationEffects[0];
		final double va = innovationEffects[1];
		double part = 0;
		for(int i=1;i<=market.getConsumerPlanningHorizon();i++){
			part+=(iv+va+(total*c)-k)*Math.pow((1+market.getInterestRate()),0-i);
		}
		final double benefit = part-p;
		return benefit;
	}



	/**
	 * This method parses a list of values and returns the index of the highest
	 * value.
	 * 
	 * @param values
	 *            the list of values
	 * 
	 * @return the index of the highest value
	 */
	private int findBest(final double[] values){
		int best = 0;
		final java.util.Random rand = new Random();
		for(int i=0;i<values.length;i++){
			if(values[i]>values[best]){
				best = i;
			}
			else if(values[i]==values[best]){
				if(rand.nextBoolean()){
					best = i;
				}
			}
		}
		if(values[best]>0){
			return best;
		}
		else{
			return -1;
		}
	}


	/**
	 * This method calculates the "Return on Investment" for this timestep.
	 * 
	 * @return a double representing the return on investment
	 */
	private double generateROI(){
		final double iv = bm.calculateValuation();
		final double k = bm.getSubscriptionFee(sp, true);
		int nofCons;
		int total;
		if(netEffectsAreLocal){
			nofCons = friends.size();
			total = nofFriendsUsingSP(sp.getUniqueID());
		}
		else{
			nofCons = market.getNofConsumers();
			total = bm.getNofSubscribers(sp, true);
		}
		final double[] innovationEffects = bm.calculateInnovationsEffects(sp, iv, nofCons);
		final double c = innovationEffects[0];
		final double va = innovationEffects[1];
		return iv+va+(total*c)-k;
	}


	/**
	 * Method to get the value this consumer calculated for the specified service provider 
	 * (used for statistical purposes)
	 * @param sp The service provider
	 * @return The value of the service provider
	 */
	public double getCalculatedValue(final int sp){
		return memory.getCalculatedProviderValue(sp);
	}


	/**
	 * Method to get a list of the unique IDs for the friends of this consumer
	 * @return  a list of the unique IDs
	 */
	public int[] getFriends(){
		return friends.toIntegerArray();
	}
	
	/**
	 * Method to get the number of friends of this consumer
	 * @return the number of friends
	 */
	public int getNofFriends(){
		return friends.size();
	}


	/**
	 * Getter method for the unique ID of the consumer.
	 * @return         the unique ID of the consumer
	 */
	public int getUniqueID(){
		return uniqueID;
	}


	/**
	 * Method to check whether or not there is room for more friends.
	 * 
	 * @return true if the friend network is full, else false
	 */
	public boolean hasEnoughFriends(){
		return friends.size()>=bm.getMaxFriends();
	}

	/**
	 * Method to check whether this consumer is friend with a specific other
	 * consumer.
	 * 
	 * @param friendID
	 *            the ID of the friend to check
	 * 
	 * @return true if the consumer is friend with the specified consumer
	 */
	public boolean isFriend(final int friendID){
		return friends.contains(friendID);
	}


	public boolean isUsing(final String spUniqueID){
		if(sp!=null){
			return sp.getUniqueID().equalsIgnoreCase(spUniqueID);
		} else {
			return false;
		}
	}


	/**
	 * Method for getting whether or not this consumer knows the specified innovation
	 * @param unique the unique ID of the innovation
	 * @return true if the consumer knows the innovation
	 */
	public boolean knowsInnovation(final String unique) {
		if(memory.knowsInnovation(unique)){
			return true;
		}
		else{
			return false;
		}
	}

	
	private int nofFriendsUsingSP(final String spUniqueID){
		int nof = 0;
		for(int i=0;i<friends.size();i++){
			final int friendId = friends.get(i);
			if(market.getConsumer(friendId).isUsing(spUniqueID)) {
				nof++;
			}
		}
		return nof;
	}

	
	/* (non-Javadoc)
	 * @see edu.ntnu.ecosim.agents.Agent#performAction()
	 */
	@Override
	public void performAction() {
		hasChangedProvider = false;
		memory.eraseOldMemory(market.getTime());
		//If the investment is "paid back"
		if(returnOnInvestment>=0){
			bm.searchForNewProviders();
			final ArrayList<String> knownProviders = memory.getKnownProviders();
			final int nofprov = market.getNofProviders();
			final double benvalues[] = new double[nofprov];
			for (int i = 0; i < nofprov; i++) {
				final ServiceProvider spro = market.getServiceProvider(i);
				double value = 0; 
				boolean knowsProvider = knownProviders.contains(spro.getUniqueID());
				if(providerInfoIsGlobal) {
					knowsProvider = true;
				}
				if(knowsProvider){
					//We assume that service providers allready tried out are not to be tried again
					// for a period since these have a "bad reputation" for the consumer
					boolean isCurrent = false;
					if(sp!=null) {
						isCurrent=spro.getUniqueID().equals(sp.getUniqueID());
					}
					if(isCurrent||!memory.isTried(spro.getUniqueID())){
						value = calculateBenefits(spro);
						memory.addCalculatedProviderValue(i, value);
					}
				}
				benvalues[i] = value;
			}
			final int best = findBest(benvalues);
			if (best>=0) {
				selectServiceProvider(market.getServiceProvider(best));
			}
		}
		else{
			returnOnInvestment+=generateROI();
		}
	}

	/**
	 * Method to check whether the consumer has changed service provider this round 
	 * @return true if the service provider is changed
	 */
	public boolean providerChanged(){
		return hasChangedProvider;
	}

	
	/**
	 * Method used to receive information about provider from a friend
	 * @param sproUniqueID the unique ID of the service provider
	 * @param persuation the rate of persuation the friend has on this consumer
	 * @param innovations a list of known innovations for this provider
	 */
	public void receiveProviderInformation(final String sproUniqueID, final int persuation, final HashMap<String, String> innovations){
		memory.addKnowledge(sproUniqueID, persuation);
		memory.addInnovationKnowledge(innovations);
	}


	/**
	 * This method is selecting the new service provider and performs the
	 * necessary actions related to this selection.
	 * 
	 * @param newSp
	 *            the new service provider
	 */
	private void selectServiceProvider(final ServiceProvider newSp){
		announceProvider(newSp.getUniqueID());
		if(sp!=null){
			if(sp.getUniqueID().equals(newSp.getUniqueID())){
				return;
			}
			else{
				sp.removeSubscriber();
				hasChangedProvider = true;
			}
		}
		sp = newSp;
		sp.addSubscriber();
		memory.addTriedProvider(newSp.getUniqueID(),market.getTime());
		returnOnInvestment = 0-newSp.getBindingFee();
	}
	
	/**
	 * Method to be used to sort the friend list of this consumer in
	 * increasing order
	 */
	public void sortFriendList() {
		friends.sortList();
		
	}


	/**
	 * By using this method, the empty slots of the friend list are removed. This
	 * will save some memory if there are many consumers in the market
	 */
	public void trimFriendList(){
		friends.trimList();
	}
	
}
