/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.consumers;

import java.util.Arrays;


/**
 * The Class FriendList.
 * 
 * This class is meant as an alternative to ArrayList to save memory. Since the number of consumers can be quite
 * extensive, this class is intended to reduce the memory usage of each consumer.
 */
public class FriendList {

	/** The number of friends. */
	private int nofFriends;
	
	/** The friends. */
	private int[] friends;
	
	/**
	 * The Constructor of a friend list.
	 * 
	 * @param maxFriends
	 *            the maximum number of friends allowed in this friendlist
	 */
	protected FriendList(final int maxFriends){
		friends = new int[maxFriends];
		nofFriends = 0;
	}
	
	/**
	 * Add one friend to the list.
	 * 
	 * @param friend
	 *            the friend
	 * 
	 * @throws IndexOutOfBoundsException
	 *            thrown if the list is full
	 */
	protected void add(final int friend) throws IndexOutOfBoundsException{
		if(nofFriends>=friends.length){
			throw new IndexOutOfBoundsException("Could not add new friend... the list is full. Contains "+nofFriends+" friends allready, and the size is "+friends.length);
		}
		friends[nofFriends] = friend;
		nofFriends++;
	}
	
	/**
	 * Method saying whether or not the list contains the specified friend
	 * 
	 * @param friend
	 *            the friend
	 * 
	 * @return true, if contains
	 */
	protected boolean contains(final int friend){
		for(int i=0;i<nofFriends;i++){
			if(friends[i]==friend) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Getter method for the friendlist.
	 * 
	 * @param place
	 *            the place where the friend is located
	 * 
	 * @return an int with the ID of the consumer
	 * 
	 * @throws IndexOutOfBoundsException
	 *             thrown if the place is out of bounds
	 */
	protected int get(final int place) throws IndexOutOfBoundsException{
		if(place>=nofFriends){
			throw new IndexOutOfBoundsException("The list did not contain "+place+"friends.");
		}
		else{
			return friends[place];
		}
	}
	
	/**
	 * Method to get the size of the list
	 * 
	 * @return the number of friends in the list
	 */
	protected int size(){
		return nofFriends;
	}
	
	
	/**
	 * Method used to sort the list of friends.
	 */
	protected void sortList(){
		Arrays.sort(friends);
	}
	
	/**
	 * Method to get the list of friends represented as an integer array containing the 
	 * unique IDs of the friends
	 * @return an integer array containing the uniqe ids of the friends
	 */
	protected int[] toIntegerArray(){
		return friends;
	}
	
	/**
	 * Method to trim the friend list, removing unused length of the array
	 *
	 */
	protected void trimList(){
		final int[] newList = new int[nofFriends];
		for(int i=0;i<nofFriends;i++){
			newList[i] = friends[i];
		}
		friends = newList;
	}
	
}
