/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.consumers;

import edu.ntnu.ecosim.basicfunctionality.Config;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This class represents the memory of the consumers.
 * @author       Martin
 */
public class Memory {
	/** The memory time defines how long information should be kept in the memory. */
	private int memorytime;
	/** This hashmap collects the providers allready tried out, and also the time at which the provider was tried. */
	private HashMap<String, Integer> triedProviders;
	/** This hashmap collects the known providers, and also an integer [1,100] that says how well known the provider is to the consumer */
	private HashMap<String, Integer> knownProviders;
	/** The known innovations are the innovations the consumer has information about */
	private HashMap<String, String> knownInnovations;
	/**The calculated provider values are the calculated benefits the consumer calculated for each provider this round (time step) */
	private HashMap<Integer, Double> calculatedProviderValue;
	
	/**
	 * Constructor of the memory.
	 */
	protected Memory(){
		triedProviders = new HashMap<String,Integer>();
		knownProviders = new HashMap<String, Integer>();
		calculatedProviderValue = new HashMap<Integer, Double>();
		knownInnovations = new HashMap<String, String>();
		try{
			memorytime = Integer.parseInt(Config.getInstance().getProperty("ecosim", "Consumers.MemoryTime"));
		}
		catch(final Exception e){
			memorytime = 12;
		}
	}

	
	/**
	 * Method to add a value to the "calculatedProviderValue" list
	 * @param sp the service providers place in the market list
	 * @param value the value of the provider
	 */
	protected void addCalculatedProviderValue(final int sp, final double value){
		calculatedProviderValue.put(Integer.valueOf(sp), Double.valueOf(value));
	}
	
	/**
	 * Method that may be used to add knowledge about a range of innovations (represented through a HashMap)
	 * @param newInnovations a HashMap containing the new innovations
	 */
	protected void addInnovationKnowledge(final HashMap<String,String> newInnovations){
		knownInnovations.putAll(newInnovations);
	}
	
	
	/**
	 * Method to add knowledge about an innovation
	 * @param uniqueID the unique ID of the innovation
	 * @param spUniqueID the unique ID of the service provider providing the innovation
	 */
	protected void addInnovationKnowledge(final String uniqueID, final String spUniqueID){
		if(!knownInnovations.containsKey(uniqueID)){
			knownInnovations.put(uniqueID, spUniqueID);
		}
	}
	
	
	/**
	 * Method used to add knowledge about a service provider.
	 * 
	 * @param persuation
	 *            the persuation force of this knowledge
	 * @param uniqueID
	 *            The unique ID of the service provider
	 */
	protected void addKnowledge(final String uniqueID, final int persuation){
		if(knownProviders.containsKey(uniqueID)){
			int totalpersuation = knownProviders.get(uniqueID)+persuation;
			if(totalpersuation>=100) {
				totalpersuation=100;
			}
			knownProviders.put(uniqueID, totalpersuation);
		}
		else{
			knownProviders.put(uniqueID, persuation);
		}
	}
	
	
	
	/**
	 * Method used when a new provider is tried out.
	 * 
	 * @param time
	 *            the time at which the provider was chosen
	 * @param uniqueID
	 *            the unique ID of the tried provider
	 */
	protected void addTriedProvider(final String uniqueID, final int time){
		triedProviders.put(uniqueID, time);
	}
	
	/**
	 * Method to erase memory older than the "memory time".
	 * 
	 * @param time The current time
	 */
	protected void eraseOldMemory(final int time){
		final Iterator iterator = triedProviders.entrySet().iterator();
		final ArrayList<String> removelist = new ArrayList<String>();
		while(iterator.hasNext()){
			final Map.Entry entry = (Map.Entry)iterator.next();
			if((time-(Integer)entry.getValue())>=memorytime){
				removelist.add((String)entry.getKey());
			}
		}
		for(final String item : removelist){
			triedProviders.remove(item);
		}
	}
	
	
	/**
	 * Mehtod to get the calculated value of a specified service provider
	 * @param sp the service providers place in the market list
	 * @return the value
	 */
	protected double getCalculatedProviderValue(final int sp){
		if(calculatedProviderValue.containsKey(Integer.valueOf(sp))){
			return calculatedProviderValue.get(Integer.valueOf(sp));
		} else {
			return 0;
		}
	}
	
	/**
	 * Method used to get the list of all the innovations this memory contains about a specified provider
	 * @param spUniqueID the unique ID of the service provider
	 * @return a HashMap containing the unique IDs of the innovations and the unique ID of the service provider they belong to
	 */
	protected HashMap<String, String> getInnovationsOfProvider(final String spUniqueID){
		final HashMap<String,String> toReturn = new HashMap<String,String>();
		final Iterator iterator = knownInnovations.entrySet().iterator();
		while(iterator.hasNext()){
			final Map.Entry entry = (Map.Entry)iterator.next();
			if(((String)entry.getValue()).equals(spUniqueID)){
				toReturn.put((String)entry.getKey(), spUniqueID);
			}
		}
		return toReturn;
	}
	
	/**
	 * Method to get the list of known providers.
	 * @return       the list of known providers
	 */
	protected ArrayList<String> getKnownProviders(){
		final Iterator iterator = knownProviders.entrySet().iterator();
		final ArrayList<String> returnlist = new ArrayList<String>();
		while(iterator.hasNext()){
			final Map.Entry entry = (Map.Entry)iterator.next();
			if((Integer)entry.getValue()>=100){
				returnlist.add((String)entry.getKey());
			}
		}
		return returnlist;
	}
	
	/**
	 * This method can be used to check whether or not a provider is tried out
	 * earlier
	 * 
	 * @param uniqueID
	 *            the unique id of the provider
	 * 
	 * @return true if the provider is tried before
	 */
	protected boolean isTried(final String uniqueID){
		return triedProviders.containsKey(uniqueID);
	}
	
	/**
	 * Method to check whether this memory contains information about an innovation
	 * @param uniqueID the unique id of the innovation
	 * @return true if the memory contains the innovation
	 */
	protected boolean knowsInnovation(final String uniqueID){
		return knownInnovations.containsKey(uniqueID);
	}
	
	
}
