/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.serviceproviders;

import edu.ntnu.ecosim.exceptions.NotEnoughDataException;
import java.util.ArrayList;

/**
 * The Class Memory represents the memory of a service provider.
 * @author  martin.andreas.borke
 */
public class Memory {
	
	/** The number of subscribers for each time step. */
	private ArrayList<Integer> subscribers;
	
	/** The profit for each time step. */
	private ArrayList<Integer> profit;

	/** The number of new subscribers this period. */
	private int periodNewSubscribers;
	
	/** The number of lost subscribers this period. */
	private int periodLostSubscribers;
	
	/** The subscription earnings this period. */
	private double subscriptionEarnings;
	
	/** The bind fee earnings this period. */
	private double bindFeeEarnings;

	/**
	 * The Constructor.
	 */
	protected Memory(){
		subscribers = new ArrayList<Integer>();
		profit = new ArrayList<Integer>();
		profit.add(Integer.valueOf(0));
		periodNewSubscribers = 0;
		periodLostSubscribers = 0;
		subscriptionEarnings = 0;
		bindFeeEarnings = 0;
	}

	/**
	 * Adds bind fee earnings.
	 * 
	 * @param earn
	 *            the earnings to add
	 */
	protected void addBindFeeEarnings(final double earn){
		bindFeeEarnings+=earn;
	}

	/**
	 * Method to add a lost subscriber to the calculations
	 */
	protected void addLostSubscriber(){
		periodLostSubscribers++;
	}

	/**
	 * Adds a new subscriber
	 */
	protected void addNewSubscriber(){
		periodNewSubscribers++;
	}

	/**
	 * Adds subscription earnings.
	 * 
	 * @param earn
	 *            the earnings
	 */
	protected void addSubscriptionEarnings(final double earn){
		subscriptionEarnings+=earn;
	}

	/**
	 * Calculates the arpu for this round.
	 * 
	 * @return the ARPU (Average Revenue Per User)
	 * 
	 * @throws NotEnoughDataException
	 *             if it is to early to calculate ARPU
	 */
	protected int calculateArpu() throws NotEnoughDataException{
		if(subscribers.size()>=2){
			double arpu = 0;
			final double roundProfit = profit.get(profit.size()-1)-profit.get(profit.size()-2);
			final int divideSubscribers = subscribers.get(subscribers.size()-1);
			if(divideSubscribers>0){
				arpu = roundProfit/divideSubscribers;
			} 
			return (int)arpu;
		}
		else if(subscribers.size()==1){
			double arpu = 0;
			final double roundProfit = profit.get(profit.size()-1);
			final int divideSubscribers = subscribers.get(subscribers.size()-1);
			if(divideSubscribers>0){
				arpu = roundProfit/divideSubscribers;
			} 
			return (int)arpu;
		}
		else{
			throw new NotEnoughDataException();
		}
	}


	/**
	 * Calculates the customer churn rate.
	 * 
	 * @return the customer churn rate
	 * 
	 * @throws NotEnoughDataException
	 *             If it is too early to calculate customer churn
	 */
	protected int calculateChurnRate() throws NotEnoughDataException{
		if(subscribers.size()>=3){
			final double s1 = subscribers.get(subscribers.size()-1)-subscribers.get(subscribers.size()-2);
			final double s2 = subscribers.get(subscribers.size()-2)-subscribers.get(subscribers.size()-3);
			double churn = 0;
			if(s2!=0){
				churn = (s2-s1)/s2;
			}
			final int pctChurn = (int)(churn*100);
			return pctChurn;
		}
		else{
			throw new NotEnoughDataException();
		}
	}


	/**
	 * Flush memory from "short time"(periodic) to long time memory.
	 */
	protected void flushMemory(){
		final int subs = subscribers.get(subscribers.size()-1)+periodNewSubscribers-periodLostSubscribers;
		subscribers.add(Integer.valueOf(subs));
		final int prof = profit.get(profit.size()-1)+(int)Math.round(subscriptionEarnings+bindFeeEarnings);
		profit.add(Integer.valueOf(prof));
		periodNewSubscribers = 0;
		periodLostSubscribers = 0;
		subscriptionEarnings = 0;
		bindFeeEarnings = 0;
	}

	/**
	 * Gets the number of subscribers from last round
	 * 
	 * @return the number of subscribers last round
	 */
	protected int getLastSubscribers(){
		return subscribers.get(subscribers.size()-1);
	}

	/**
	 * Gets the lost subscribers this round
	 * 
	 * @return the lost subscribers
	 */
	protected int getLostSubscribers() {
		return periodLostSubscribers;
	}

	/**
	 * Gets the new subscribers this round
	 * 
	 * @return the new subscribers
	 */
	protected int getNewSubscribers(){
		return periodNewSubscribers;
	}

	/**
	 * Gets the total profit of this provider
	 * @return  the profit
	 */
	protected int getProfit(){
		return profit.get(profit.size()-1)+(int)Math.round(subscriptionEarnings+bindFeeEarnings);
	}

	/**
	 * Gets the round profit.
	 * 
	 * @return the round profit
	 */
	protected double getRoundProfit() {
		return subscriptionEarnings+bindFeeEarnings;
	}

	/**
	 * Gets the total subscribers.
	 * 
	 * @return the total subscribers
	 */
	protected int getTotalSubscribers(){
		return subscribers.get(subscribers.size()-1)+periodNewSubscribers-periodLostSubscribers;
	}

	/**
	 * Register initial subscribers.
	 * 
	 * @param subs
	 *            the number of subscribers
	 */
	protected void registerInitialSubscribers(final int subs){
		subscribers.add(Integer.valueOf(subs));
	}
	
	
}
