/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.serviceproviders;

import edu.ntnu.ecosim.agents.Agent;
import edu.ntnu.ecosim.agents.serviceproviders.innovation.Innovation;
import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.environment.Market;
import edu.ntnu.ecosim.exceptions.NoSuchStrategyException;
import java.util.ArrayList;
import java.util.UUID;

/**
 * This is the class representing the service provider agent.
 * @author           Martin
 */
public class ServiceProvider extends Agent{

	/** A list of the innovations for this provider */
	private ArrayList<Innovation> innovations;
	
	/**The name of this provider */
	private String name;

	/**The unique ID of the provider. */
	private String uniqueID;

	/**The memory of this provider */
	private Memory memory;
	/**The strategy of this provider*/
	private Strategy strategy;

	/**The subscription fee for this provider */
	private double subscriptionFee;
	/** The binding fee for this provider */
	private double bindingFee;
	
	/**
	 * The constructor of service providers
	 * 
	 * @param name The name of the provider
	 * @param initsub The number of initial subscribers for this service provider
	 * @param subFee The subscription fee
	 * @param bindFee The binding fee
	 * @param market The market the provider operates within
	 * @param strat The strategy of the provider
	 * @param innostr The innovation strategy of the provider
	 * @param innotype The innovation type of the provider
	 * @param ecofl The economic flexibility that the provider has
	 * @throws NoSuchStrategyException if the strategy is not recognized
	 */
	public ServiceProvider(final String name, final int initsub, final double subFee, final double bindFee, final Market market, final String strat, final int innostr, final int innotype, final int ecofl)
		throws NoSuchStrategyException{
		super(market);
		memory = new Memory();
		innovations = new ArrayList<Innovation>();		
		subscriptionFee = subFee;
		bindingFee = bindFee;
		uniqueID = UUID.randomUUID().toString();
		this.name =  name;
		memory.registerInitialSubscribers(initsub);
		strategy = new Strategy(this, memory, strat, ecofl, innostr, innotype);
		int steppct = 50;
		int countdown = (initsub/steppct);
		if(initsub<100) {
			steppct = 20;
		}
		System.out.println("Creating "+initsub+" subscribers for "+name);
		for(int i=0;i<initsub;i++){
			countdown--;
			market.addConsumer(this);
			if(countdown<=0){
				System.out.print("#");
				countdown = (initsub/steppct);
			}
		}
		System.out.println("100% \n");
		Runtime.getRuntime().gc();
	}


	/**
	 * Adds an innovation to the providers list of innovations
	 * @param i the innovation
	 */
	public void addInnovation(final Innovation i){
		innovations.add(i);
		if(i.getType()==Innovation.NETEFFADDING){
			Log.getInstance().writeToLog("At time "+market.getTime()+", "+name+ " created new innovation with network effect "+i.getNetEffect(), Log.INFO);
		}
		else{
			Log.getInstance().writeToLog("At time "+market.getTime()+", "+name+" created a value adding innovation which adds "+((int)(i.getValueAdd()*100))+"% to the subscription value.", Log.INFO);
		}
	}
	
	/**
	 * Adds a subscriber to the total number of subscribers.
	 */
	public void addSubscriber(){
		memory.addNewSubscriber();
		memory.addBindFeeEarnings(bindingFee);
	}
	
	/**
	 * Method to calculate the ARPU of the provider
	 * @return the ARPU
	 */
	public double calculateArpu(){
		final double roundProfit = memory.getRoundProfit();
		final int divideSubscribers = memory.getTotalSubscribers()+memory.getLostSubscribers();
		if(divideSubscribers>0){
			return roundProfit/divideSubscribers;
		} else {
			return 0;
		}
	}
	
	/**
	 * Calculates the customer churn rate of the provider for this round
	 * @return the churn rate
	 */
	public double calculateChurnRate(){
		return (double)(memory.getTotalSubscribers()-memory.getLastSubscribers())/memory.getLastSubscribers();
	}

	/**
	 * Method used to compare this providers costs to the average subscription costs in the market. 
	 * The method calculates the average susbscription costs.
	 * @return the avg. subscription costs
	 */
	protected int getAvgSubsCostInMarket(){
		double sum = 0;
		for(int i=0;i<market.getNofProviders();i++){
			sum+=market.getServiceProvider(i).getSubscriptionFee();
		}
		return (int)(sum/market.getNofProviders());
	}

	/**
	 * Method used to get the binding fee of the provider
	 * @return      the bindingFee
	 */
	public double getBindingFee(){
		return bindingFee;
	}

	/**
	 * Method to get an innovation from this providers list of innovations
	 * @param place the place in the list the innovation is located at
	 * @return the innovation
	 */
	public Innovation getInnovation(final int place){
		return innovations.get(place);
	}

	/**
	 * Gets the name of the provider
	 * @return         the name
	 */
	public String getName(){
		return name;
	}

	/**
	 * Gets the number of new subscribers.
	 * 
	 * @return the number of new subscribers
	 */
	public int getNewSubscribers(){
		return memory.getNewSubscribers();
	}

	/**
	 * Gets the number of consumers in the market
	 * @return the number of consumers
	 */
	protected int getNofConsumersInMaret(){
		return market.getNofConsumers();
	}

	/**
	 * Gets the number of innovations by this service provider
	 * @return the number of innovations
	 */
	public int getNofInnovations(){
		return innovations.size();
	}

	
	/**
	 * Gets the number of subscribers.
	 * 
	 * @return the number of subscribers
	 */
	public int getNofSubscribers(){
		return memory.getTotalSubscribers();
	}
	
	/**
	 * Gets the total profit of this provider.
	 * 
	 * @return the profit
	 */
	public int getProfit() {
		return memory.getProfit();
	}
	
	/**
	 * Gets the profit for this round
	 * @return the profit
	 */
	public double getRoundProfit(){
		return memory.getRoundProfit();
	}

	/**
	 * Gets the subscription fee of this provider
	 * @return      the subscriptionFee
	 */
	public double getSubscriptionFee(){
		return subscriptionFee;
	}

	/**
	 * Gets the unique ID of the provider.
	 * @return         the unique ID
	 */
	public String getUniqueID(){
		return uniqueID;
	}
	
	/* (non-Javadoc)
	 * @see edu.ntnu.ecosim.agents.Agent#performAction()
	 */
	@Override
	public void performAction(){
		memory.flushMemory();		
		memory.addSubscriptionEarnings(Math.round((memory.getTotalSubscribers()*subscriptionFee)));
		strategy.applyStrategy();
	}
	
	/**
	 * Removes one subscriber from the total number of subscribers this service
	 * provider has. Used when a subscriber changes to another provider.
	 */
	public void removeSubscriber(){
		memory.addLostSubscriber();
	}
	
	/**
	 * Sets the binding fee for this provider
	 * @param bf      the new binding fee of the service provider
	 */
	protected void setBindingFee(final double bf){
		bindingFee = bf;
	}

	/**
	 * Sets the subscription fee for the provider
	 * @param sf      the new subscription fee
	 */
	public void setSubscriptionFee(final double sf){
		subscriptionFee = sf;
	}
	
	
}
