/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.serviceproviders;

import edu.ntnu.ecosim.agents.serviceproviders.innovation.Innovation;
import edu.ntnu.ecosim.exceptions.NoSuchStrategyException;
import edu.ntnu.ecosim.exceptions.NotEnoughDataException;

/**
 * The Class Strategy represents the companies strategy.
 * @author  martin.andreas.borke
 */
public class Strategy {

	//Parameters representing different kinds of strategies
	public final static int NONE = 0;
	public final static int INNOVATIVE = 1;
	public final static int REACTIVE = 2;
	public final static int INNOREACTIVE = 3;

	//Parameters used for internal memory of last chosen strategy (if reactive)
	private final static int NO_STRATEGY = 0;
	private final static int INCREASE_SUBS = 1;
	private final static int INCREASE_REV = 2;

	/**The service provider this strategy belongs to*/
	private ServiceProvider sp;
	/**The memory of the service provider */
	private Memory memory;
	/**The innovaiton lab developing innovations for this provider  */
	private InnovationLab innoLab;
	/**The innovative strength of the provider */
	private int innovativeStrength;
	/**The economic flexibility of the provider */
	private int economicFlexibility;
	/**The provider type (refers to the strategies) */
	private int providertype;
	/**The current economic strategy */
	private int currentStrategy;
	/**The last bind fee */
	private double oldBindFee;
	/**A timing parameter for pricing strategies */
	private int bindFeeDumpCounter;
	/**Whether or not this provider shall have predator behavior */
	private boolean isPredator;


	/**
	 * Constructor for the strategy
	 * @param sp the service provider the strategy belongs to 
	 * @param mem the memory of the service provider
	 * @param strat the strategy to use
	 * @param ecofl the economic flexibility of the provider
	 * @param innostr the innovative strength of the provider
	 * @param innotype the innovation type this provider develops
	 * @throws NoSuchStrategyException if the strategy string is not recognized
	 */
	protected Strategy(final ServiceProvider sp, final Memory mem, final String strat, final int ecofl, final int innostr, final int innotype) throws NoSuchStrategyException{
		isPredator = false;
		if(ecofl>10){
			isPredator = true;
			economicFlexibility = 10;
		}
		else{
			economicFlexibility = ecofl;
		}
		innovativeStrength = innostr;
		this.sp = sp;
		parseStratString(strat);
		memory = mem;
		currentStrategy = NO_STRATEGY;
		innoLab = new InnovationLab(innovativeStrength, innotype);
		bindFeeDumpCounter = 0;
		oldBindFee = 0;
	}

	/**
	 * Method used each time the strategy module shall react
	 *
	 */
	protected void applyStrategy(){
		switch(providertype){
		case NONE:
			break;
		case INNOVATIVE:
			innovate();
			break;
		case REACTIVE:
			react();
			break;
		case INNOREACTIVE:
			innovate();
			react();
			break;
		}
	}


	/**
	 * This method tries to select the best reactive strategy based on arpu and churn rate
	 *
	 */
	private void findBestStrategy(){
		try{
			final int arpu = memory.calculateArpu();
			final int churn = memory.calculateChurnRate();
			final int avgSubsCostInMark = sp.getAvgSubsCostInMarket();
			if(churn>5){
				currentStrategy=INCREASE_SUBS;
			}
			else if(arpu<avgSubsCostInMark){
				currentStrategy = INCREASE_REV;
			}
			else if(isPredator){
				final double markShare = ((double)sp.getNofSubscribers())/sp.getNofConsumersInMaret();
				if(markShare<0.8){
					currentStrategy = INCREASE_SUBS;
				}
			}
			else{
				currentStrategy = NO_STRATEGY;
			}
		}
		catch(final NotEnoughDataException e){
			currentStrategy = NO_STRATEGY;
		}
	}


	/**
	 * Method used when the service provider shall innovate
	 *
	 */
	private void innovate(){
		final Innovation i = innoLab.innovate();
		if(i!=null){
			sp.addInnovation(i);
		}
	}



	/**
	 * Method used for parsing of a string representing a strategy
	 * @param strat the int representation of the strategy
	 * @throws NoSuchStrategyException if the strategy is not recognized
	 */
	private void parseStratString(final String strat) throws NoSuchStrategyException{
		if(strat.equalsIgnoreCase("NONE")) {
			providertype = NONE;
		} else if(strat.equalsIgnoreCase("INNOVATIVE")) {
			providertype = INNOVATIVE;
		} else if(strat.equalsIgnoreCase("REACTIVE")) {
			providertype = REACTIVE;
		} else if(strat.equalsIgnoreCase("INNOREACTIVE")) {
			providertype =  INNOREACTIVE;
		} else{
			providertype = NONE;
			throw new NoSuchStrategyException("Could not find strategy: "+strat);
		}
	}

	/**
	 * Method used if the service provider shall react (that is: use economic 
	 * means as a strategy)
	 */
	private void react(){
		findBestStrategy();
		final double sf = sp.getSubscriptionFee();
		switch(currentStrategy){
		case INCREASE_REV:
			sp.setBindingFee(oldBindFee);
			bindFeeDumpCounter = 0;
			break;
		case INCREASE_SUBS:
			final double priceDecrease = (double)(economicFlexibility)/200; 
			sp.setSubscriptionFee(sf-(sf*priceDecrease));
			if(bindFeeDumpCounter<=0){
				if(sp.getBindingFee()!=0) {
					oldBindFee = sp.getBindingFee();
				}
				sp.setBindingFee(0);
				bindFeeDumpCounter = 3;
			}
			else{
				bindFeeDumpCounter--;
			}
			break;
		default: 
			break;
		}
	}

}
