/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.basicfunctionality;

import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import org.w3c.dom.*;


/**
 * This class is used to read the cofiguration file and put it into an accessible hashtable. 
 * To get a property, the Config.getInstance().getProperty("ecosim","property") method is 
 * used.
 */
public class Config {

	/** The CONFIGPATH. */
	private static String CONFIGPATH;

	/** The _config singleton variable */
	private static Config _config = new Config();

	/**Controller variable to avoid setting the config path several times */
	private static boolean configPathSet;


	/**
	 * Method to get an instance of the configuration file
	 * @return The instance
	 */
	public static Config getInstance() {
		if (_config == null) {
			synchronized (Config.class) {
				if (_config == null) {
					_config = new Config();
				}
			}
		}
		return _config;
	}

	/** The hashtable containing the modules. */
	private Hashtable<String,ConfigProperties> _modules;

	/** The xml reader. */
	private XMLConfigFile _xmlReader;

	/**
	 * The parameter holding the time last modified.
	 */
	private long lastModified = 0;

	/**
	 * The Constructor.
	 */
	private Config() {
		_modules = null;
		_xmlReader = null;
		_modules = new Hashtable<String, ConfigProperties>();
		_xmlReader = new XMLConfigFile();
		configPathSet = false;
	}


	/**
	 * Method called to fetch properties from the configuration
	 * 
	 * @param useCache
	 *            Whether the cached variable should be used
	 * @param moduleName
	 *            Name of module
	 * 
	 * @return The properties of the module with the given name
	 */
	private ConfigProperties getModule(final String moduleName, final boolean useCache) {
		ConfigProperties moduleProperties = null;
		if (useCache) {
			moduleProperties = _modules.get(moduleName);
		}
		if (moduleProperties == null) {
			final Document doc = _xmlReader.readConfFile(moduleName);
			if (doc != null) {
				final Element root = doc.getDocumentElement();
				for (Node node = root;
				node != null;
				node = node.getNextSibling()) {
					if ((node instanceof Element)
							&& node.getNodeName().equals("module")) {
						moduleProperties = new ConfigProperties();
						final NamedNodeMap attrib = node.getAttributes();
						final Node thisNode = attrib.getNamedItem("name");
						thisNode.getNodeValue();
						moduleProperties = traverseDOM((Element)node);
						if (moduleProperties != null) {
							synchronized (this) {
								if (!_modules.containsKey(moduleName)) {
									_modules.put(moduleName, moduleProperties);
								} else {
									_modules.remove(moduleName);
									_modules.put(moduleName, moduleProperties);
								}
							}
						}
					}
				}

			}
		}
		return moduleProperties;
	}

	/**
	 * Method to get a property from the configuration. If the configuration file is
	 * changed, the new properties are read.
	 * 
	 * @param propertyName
	 *            Name of the required property, for instance "Log.LogLevel"
	 * @param moduleName
	 *            Name of the module
	 * 
	 * @return A string representing the value of the property
	 */
	public synchronized String getProperty(final String moduleName, final String propertyName) throws PropertyNotFoundException{
		boolean useCache;
		final File newPropertyFile =
			new File(
					CONFIGPATH
					+ System.getProperty("file.separator")
					+ "ecosim.xml.new");
		if (lastModified < _xmlReader.getLastModified(moduleName)) {
			useCache = false;
		} else if (newPropertyFile.exists()) {
			importNewProperties();
			useCache = false;
		} else {
			useCache = true;
		}
		final ConfigProperties moduleProperties = getModule(moduleName, useCache);
		if (moduleProperties != null) {
			return moduleProperties.getProperty(propertyName);
		} else {
			throw new PropertyNotFoundException("Could not find property: "+propertyName);
		}
	}

	/**
	 * Gets the configuration properties of a sub module.
	 * 
	 * @param subModuleName
	 *            the sub module name
	 * @param moduleName
	 *            the module name
	 * 
	 * @return the configuration properties of the submodule
	 */
	public ConfigProperties getSubModule(
			final String moduleName,
			final String subModuleName) {
		final ConfigProperties moduleProperties = getModule(moduleName, true);
		if (moduleProperties != null) {
			return moduleProperties.getSubModule(subModuleName);
		} else {
			return null;
		}
	}

	/**
	 * Method to switch between new and old configuration settings
	 */
	private synchronized void importNewProperties() {
		System.out.println("Config file changed, reading new properties...");
		final File newPropertyFile =
			new File(
					CONFIGPATH
					+ System.getProperty("file.separator")
					+ "ecosim.xml.new");
		final File oldPropertyFile =
			new File(
					CONFIGPATH
					+ System.getProperty("file.separator")
					+ "ecosim.xml");

		try {
			oldPropertyFile.delete();
			final FileInputStream streamIn = new FileInputStream(newPropertyFile);
			final FileOutputStream streamOut = new FileOutputStream(oldPropertyFile);
			int c;
			while ((c = streamIn.read()) != -1) {
				streamOut.write(c);
			}
			streamIn.close();
			streamOut.close();
			newPropertyFile.delete();

		} catch (final FileNotFoundException e) {
			System.out.println(
					"Problem reading config file:"+
					e.toString());
		} catch (final IOException e) {
			System.out.println(
					"Problems changing config file!"+
					e.toString());
		}

	}

	/**
	 * Sets the configuration file path.
	 * 
	 * @param path
	 *            the file path
	 */
	public synchronized void setConfigPath(final String path) {
		if(!configPathSet){
			CONFIGPATH = path;
			_xmlReader.setConfigPath(path);
			configPathSet = true;
		}
	}

	/**
	 * Sets the time last modified.
	 * @param l     the time last modified
	 */
	public synchronized void setLastModified(final long l) {
		lastModified = l;
	}

	/**
	 * Method for traversing the DOM file.
	 * 
	 * @param element
	 *            the element
	 * 
	 * @return the config properties
	 */
	private ConfigProperties traverseDOM(final Element element) {
		final ConfigProperties property = new ConfigProperties();
		for (Node node = element.getFirstChild();
		node != null;
		node = node.getNextSibling()) {
			if (node instanceof Element) {
				if (node.getNodeName().equals("submodule")) {
					final NamedNodeMap attrib = node.getAttributes();
					final Node thisNode = attrib.getNamedItem("name");
					final String pName = thisNode.getNodeValue();
					final ConfigProperties mProps = traverseDOM((Element)node);
					property.putProperty(pName, mProps);
				} else if (node.getNodeName().equals("property")) {
					final NamedNodeMap attrib = node.getAttributes();
					Node thisNode = attrib.getNamedItem("name");
					final String pName = thisNode.getNodeValue();
					thisNode = attrib.getNamedItem("value");
					final String pValue = thisNode.getNodeValue();
					property.putProperty(pName, pValue);
				}
			}
		}

		return property;
	}

}