/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.basicfunctionality;

import java.io.Serializable;
import java.util.*;

/**
 * Class holding the cached configuration properties, so they don't need to be read each time a property
 * is accessed.
 */
public class ConfigProperties
    implements Serializable
{

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -6080124196850889314L;

	/** The property list. */
    private Hashtable<String,Serializable> _properties;

    
    /**
	 * Constructor
	 */
    public ConfigProperties()
    {
        _properties = null;
        _properties = new Hashtable<String, Serializable>();
    }

    
    /**
	 * Method to access the elements of the configuration file
	 * 
	 * @return The elements
	 */
    public Enumeration<Serializable> elements()
    {
        return _properties.elements();
    }

    /**
	 * Method to get a "property", that is - a property of the configuration tree
	 * 
	 * @param propertyName
	 *            the name of the property you want to get
	 * 
	 * @return The value of the property
	 */
    public String getProperty(final String propertyName)
    {
        final String name[] = splitPropertyName(propertyName);
        final Object property = _properties.get(name[0]);
        if((property instanceof String) && name[1] == null) {
			return (String)property;
		}
        if((property instanceof ConfigProperties) && name[1] != null) {
			return ((ConfigProperties)property).getProperty(name[1]);
		} else {
			return null;
		}
    }

    /**
	 * Method to get a submodule
	 * 
	 * @param listName
	 *            the name of the submodule
	 * 
	 * @return the submodule
	 */
    public ConfigProperties getSubModule(final String listName)
    {
        String name[];
        name = splitPropertyName(listName);
        final Object property = _properties.get(name[0]);
        if((property instanceof ConfigProperties) && name[1] == null) {
			return (ConfigProperties)property;
		}
        if((property instanceof ConfigProperties) && name[1] != null) {
			return ((ConfigProperties)property).getSubModule(name[1]);
		} else {
			return null;
		}
    }

    
    /**
	 * Fetches the keys of the hashmap
	 * 
	 * @return Enumeration containing the keys
	 */
    public Enumeration<String> keys()
    {
        return _properties.keys();
    }

    
    /**
	 * Put a submodule of properties into the configuration
	 * 
	 * @param propertyName
	 *            the property name
	 * @param property
	 *            the properties
	 */
    public void putProperty(final String propertyName, final ConfigProperties property)
    {
        _properties.put(propertyName, property);
    }

    /**
	 * Put property into configuration.
	 * 
	 * @param propertyName
	 *            the property name
	 * @param propertyValue
	 *            the property value
	 */
    public void putProperty(final String propertyName, final String propertyValue)
    {
        _properties.put(propertyName, propertyValue);
    }

    /**
	 * Split the property name into separate values
	 * 
	 * @param name
	 *            the name
	 * 
	 * @return the string[] containing the separated values
	 */
    private String[] splitPropertyName(final String name)
    {
        final String split[] = new String[2];
        try
        {
            final StringTokenizer stz = new StringTokenizer(name, ".");
            if(stz.countTokens() > 1)
            {
                split[0] = stz.nextToken();
                split[1] = name.substring(split[0].length() + 1, name.length());
            } else
            {
                split[0] = stz.nextToken();
                split[1] = null;
            }
        }
        catch(final Exception _ex)
        {
            split[0] = "";
            split[1] = null;
        }
        return split;
    }
}