/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.basicfunctionality;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Light-weighted logging utility. 
 * @author       martin.andreas.borke
 */
public class Log {

	/** The log object (singleton). */
	private static Log logger=null;

	/** The Constant DEBUG. */
	public final static int DEBUG = 0;

	/** The Constant INFO. */
	public final static int INFO = 1;

	/** The Constant WARN. */
	public final static int WARN = 2;

	/** The Constant ERROR. */
	public final static int ERROR = 3;

	/** The Constant FATAL. */
	public final static int FATAL = 4;

	/**
	 * Gets the one instance of the log.
	 * 
	 * @return the instance
	 */
	public static Log getInstance(){
		if(logger==null){
			synchronized (Log.class) {
				if(logger==null){
					logger = new Log();
				}

			}
		}
		return logger;
	}

	/**Whether or not the log can be written to the specified directory. If false, the
	 * log-command writes to the command line
	 */
	private boolean directorypossible;

	/** The filename. */
	private String filename;

	/**The lowest level to accept log messages for*/
	private int logLevel;


	/**
	 * The Constructor.
	 */
	private Log(){
		try{
			try{
				logLevel = parseLogLevel(Config.getInstance().getProperty("ecosim", "Log.LogLevel"));
			}
			catch(final Exception e){
				System.err.println("Problems reading from config, setting log level to debug: "+e.getMessage());
				logLevel = DEBUG;
			}
			String logpath = Config.getInstance().getProperty("ecosim", "Log.LogPath");
			if(logpath.equalsIgnoreCase("current")){
				final File f = new File(".");
				final String filePath = f.getAbsolutePath();
				final int dotAt = filePath.lastIndexOf(".");
				logpath = (filePath.substring(0,dotAt))+System.getProperty("file.separator")+"logfiles";
			}
			filename = logpath+System.getProperty("file.separator")+"logfile.txt";
			final File path = new File(logpath);
			directorypossible = true;
			if(!path.exists()){
				directorypossible = path.mkdirs();
			}
			if(!directorypossible){
				System.out.println("COULD NOT CREATE LOG DIRECTORY..");
			}
			final File f = new File(filename);
			if(f.exists()){
				final Calendar cal = new GregorianCalendar();
				final String time = ""+cal.get(Calendar.HOUR_OF_DAY)+""+cal.get(Calendar.MINUTE)+""+cal.get(Calendar.SECOND);
				final String date= ""+cal.get(Calendar.DAY_OF_MONTH)+""+(1+cal.get(Calendar.MONTH))+""+cal.get(Calendar.YEAR);
				f.renameTo(new File(logpath+System.getProperty("file.separator")+"logfile"+date+"_"+time+".txt"));
			}
			writeLogFileStart();
		}
		catch(final Exception e){
			System.out.println("Log could not be created, printing to command line...");
			directorypossible = false; 
		}
	}



	/**
	 * Method to convert between textual and internal integer representation of log levels
	 * @param level The level (as string)
	 * @return The level (as int)
	 */
	public int parseLogLevel(final String level){
		try{
			if(level.equalsIgnoreCase("DEBUG")) {
				return DEBUG;
			} else if(level.equalsIgnoreCase("INFO")) {
				return INFO;
			} else if(level.equalsIgnoreCase("WARN")) {
				return WARN;
			} else if(level.equalsIgnoreCase("ERROR")) {
				return ERROR;
			} else if(level.equalsIgnoreCase("FATAL")) {
				return FATAL;
			} else {
				return DEBUG;
			}
		}
		catch(final Exception e){
			System.err.println("The config file gave a non-existing log level.. setting level to debug");
			return DEBUG;
		}
	}


	/**
	 * Method used to write initial information to the log file
	 */
	private void writeLogFileStart(){
		BufferedWriter out;
		try {
		out = new BufferedWriter(new FileWriter(filename, true));
		final Calendar cal = new GregorianCalendar();
		String h = ""+cal.get(Calendar.HOUR_OF_DAY);
		String m = ""+cal.get(Calendar.MINUTE);
		String sec = ""+cal.get(Calendar.SECOND);
		if(Integer.parseInt(h)<10) {
			h = "0"+h;
		}
		if(Integer.parseInt(m)<10) {
			m = "0"+m;
		}
		if(Integer.parseInt(sec)<10) {
			sec = "0"+sec;
		}
		final String time = ""+h+":"+m+":"+sec;
		final String date= ""+cal.get(Calendar.DAY_OF_MONTH)+"/"+(1+cal.get(Calendar.MONTH))+"/"+cal.get(Calendar.YEAR);
		out.write("#######################\n");
		out.write("LOG CREATED "+date+" "+time+"\n");
		out.write("#######################\n");
		out.close();
		} catch (final IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Write to log. (or to command line if the log could not be initialized)
	 * 
	 * @param level
	 *            the level for this log entry
	 * @param s
	 *            the string to write to the log
	 */
	public synchronized void writeToLog(final String s, final int level){
		if(directorypossible){
			try {
				final BufferedWriter out = new BufferedWriter(new FileWriter(filename, true));
				final Calendar cal = new GregorianCalendar();
				String h = ""+cal.get(Calendar.HOUR_OF_DAY);
				String m = ""+cal.get(Calendar.MINUTE);
				String sec = ""+cal.get(Calendar.SECOND);
				if(Integer.parseInt(h)<10) {
					h = "0"+h;
				}
				if(Integer.parseInt(m)<10) {
					m = "0"+m;
				}
				if(Integer.parseInt(sec)<10) {
					sec = "0"+sec;
				}
				final String time = ""+h+":"+m+":"+sec;
				out.write("\n");
				if(level>=logLevel){
					switch(level){
					case DEBUG:
						out.write("|"+time+"|"+" <DEBUG> "+s);
						break;
					case INFO:
						out.write("|"+time+"|"+" <INFO> "+s);
						break;
					case WARN:
						out.write("|"+time+"|"+" <WARN> "+s);
						break;	
					case ERROR:
						out.write("|"+time+"|"+" <ERROR> "+s);
						break;
					case FATAL:
						out.write("|"+time+"|"+" <FATAL> "+s);
						break;
					default:
						out.write("|"+time+"|"+" <NO LEVEL> "+s);
					}
				}
				out.close();
			} catch (final IOException e) {
				e.printStackTrace();
				System.out.println(s);
			}
		}
		else{
			System.out.println(s);
		}
	}


}
