/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.environment;

import edu.ntnu.ecosim.basicfunctionality.Log;
import java.util.Arrays;

/**
 * This is a class specialized for finding clusters within the
 * social network of the market. Used for analytical purposes only. 
 */
public class ClusterFinder {

	/** The market. */
	private Market market;

	/**
	 * The Constructor of a cluster finder
	 * 
	 * @param m
	 *            the market to search in
	 */
	protected ClusterFinder(final Market m){
		market = m;
	}


	/**
	 * Finds the largest clusters in the social network. The results are written to the logfile. 
	 */
	protected void findClusters(){
		System.out.println("CLUSTER SEARCH");
		System.out.println("Sorting friend lists....");
		for(int i=0;i<market.getNofConsumers();i++){
			market.getConsumer(i).sortFriendList();
		}
		System.out.println("Sort finished");
		int maxSizeCluster = 0;
		int numberOfMaxClusters = 0;
		System.out.println("Searching for clusters...");
		int counter = 0;
		int counter2 = 0;
		for(int i=0;i<market.getNofConsumers();i++){
			counter++;
			if(counter>1000){
				System.out.print(".");
				counter = 0;
				counter2++;
				if(counter2>=40){
					System.out.print("\n");
					counter2 = 0;
				}
			}
			final int consumerLargest = findLargestClusterForConsumer(i);
			if(consumerLargest>maxSizeCluster){
				maxSizeCluster = consumerLargest;
				numberOfMaxClusters = 1;
			}
			else if(consumerLargest==maxSizeCluster){
				numberOfMaxClusters++;
			}
		}
		System.out.println("\n");

		Log.getInstance().writeToLog("LARGEST CLUSTER: "+maxSizeCluster+" consumers!", Log.INFO);
		Log.getInstance().writeToLog("Number of max size clusters: "+numberOfMaxClusters, Log.INFO);
		System.out.println("Cluster search finished, results written to logfile!");
	}



	/**
	 * Find largest cluster for one specific consumer.
	 * 
	 * @param consumerID
	 *            the consumer ID of the consumer to start searching from
	 * 
	 * @return the largest cluster including this consumer
	 */
	private int findLargestClusterForConsumer(final int consumerID){
		int largest = 0;

		final int[] friendList = market.getConsumer(consumerID).getFriends();
		final int[][] clusters = new int[friendList.length][friendList.length];

		for(int i=0;i<friendList.length;i++){
			Arrays.fill(clusters[i], -1);
			clusters[i][0]=friendList[i];
			int added = 1;
			for(int j=i+1;j<friendList.length;j++){
				boolean add = true;
				for(int z=0;z<added;z++){
					if(!market.getConsumer(friendList[j]).isFriend(clusters[i][z])){
						add = false;
					}
				}
				if(add){
					clusters[i][added]=friendList[j];
					added++;
				}
			}
			final int thisMax = findMax(clusters);
			if(thisMax>largest){
				largest=thisMax;
			}
		}
		return largest;
	}


	/**
	 * Find the highest number in the table
	 * 
	 * @param list
	 *            the table containing the clusters
	 * 
	 * @return the highest number of consumers in clusters
	 */
	private int findMax(final int[][] list){
		int max = 0;
		for (int[] element : list) {
			int count = 0;
			for(int j=0;j<list.length;j++){
				if(element[j]>0){
					count++;
				}
				else{
					break;
				}
			}
			if(count>max){
				max=count;
			}
		}
		//Adding the initial node to the count!
		return max+1;
	}

}
