/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.environment;

import edu.ntnu.ecosim.agents.consumers.Consumer;
import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.basicfunctionality.Config;
import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * The consumer factory is "producing" the consumers, 
 * assuring that each consumer gets the necessary attributes 
 * and a unique ID. The consumer factory is also responsible 
 * for generating the social network.
 * @author     Martin
 */
public class ConsumerFactory {

	/** A counter to keep the IDs in order. */
	private int IDcounter;

	/** The market to produce the consumers to. */
	private Market market;

	/**
	 * Constructor of the ConsumerFactory.
	 * 
	 * @param market
	 *            the market to produce consumers to
	 */
	protected ConsumerFactory(final Market market){
		this.market = market;
		IDcounter = 0;
	}

	/**
	 * Method to analyze whether or not the "smal world effect" exists. The results are
	 * written to the logfile
	 */
	private void analyzeSmallWorldEffect(){
		final int nofCons = market.getNofConsumers();
		final int chosenCons = (int)(Math.random()*nofCons);
		final Consumer c = market.getConsumer(chosenCons);
		Hashtable<Integer, Integer> distance= new Hashtable<Integer, Integer>();
		for(int i=0;i<nofCons;i++){
			distance.put(i, Integer.valueOf(-1));
		}
		distance.put(chosenCons, 0);
		ArrayList<Integer> toCheck = new ArrayList<Integer>();
		toCheck.add(c.getUniqueID());
		System.out.println("Finding shortest paths, please wait..... \n");
		int counter = 0;
		int counter2 = 0;
		while(!toCheck.isEmpty()){
			if(counter<=0){
				System.out.print(".");
				counter = 1000;
				counter2++;
				if(counter2>=40){
					System.out.print("\n");
					counter2 = 0;
				}
			}
			counter--;
			final int next = toCheck.remove(0);
			final int[] friends = market.getConsumer(next).getFriends();
			for(final int i:friends){
				if(distance.get(i)<0){
					toCheck.add(i);
				}
				final int thisDistance = distance.get(next)+1;
				final int nodeDistance = distance.get(i);
				if(nodeDistance<0||nodeDistance>thisDistance){
					distance.put(i, thisDistance);
				}
			}
		}
		System.out.print("\n");
		final int[] nof = new int[15];

		for(int i=0;i<nof.length;i++){
			nof[i]=0;
		}

		Enumeration<Integer> keys = distance.keys();
		while(keys.hasMoreElements()){
			final Integer x = keys.nextElement();
			final int d = distance.get(x);
			if(d<nof.length&&d>=0){
				nof[d]++;
			}
		}
		String logstring = "Shortest numbers of steps (one-to-all)\n";
		for(int i=0;i<nof.length;i++){
			logstring+="             "+i+" steps: "+nof[i]+" consumers \n";
			//System.out.println(i+" steps: "+nof[i]+" consumers");
		}
		Log.getInstance().writeToLog(logstring, Log.INFO);
		System.out.println("Results written to logfile");

		distance = null;
		toCheck = null;
		keys = null;
		Runtime.getRuntime().gc();

	}


	/**
	 * Method to analyze the properties of the social network 
	 * - highest and lowest number of friends
	 * - distribution of friends
	 * 
	 * Results are written to the logfile
	 */
	private void analyzeSocialNetworks(){
		final int size = market.getNofConsumers();
		System.out.println("Analyzing social networks...");
		Log.getInstance().writeToLog("Free memory before trimming friend lists: "+Runtime.getRuntime().freeMemory(), Log.DEBUG);
		int steppct = 50;
		if(size<100) {
			steppct = 20;
		}
		int max = -1000;
		int min = -1000;
		int[] nofFriendsList;
		try {
			max = Integer.parseInt(Config.getInstance().getProperty("ecosim", "Consumers.MaxFriends"));
			min = Integer.parseInt(Config.getInstance().getProperty("ecosim", "Consumers.MinFriends"));
			nofFriendsList = new int[max-min+1];

			int countdown = (size/steppct);
			int lowest = 1000000;
			int highest = 0;
			for(int i=0;i<size;i++){
				countdown--;
				final Consumer c = market.getConsumer(i);
				final int friends = c.getNofFriends();
				c.trimFriendList();
				if(friends<lowest) {
					lowest = friends;
				}
				if(friends>highest) {
					highest = friends;
				}
				nofFriendsList[friends-min]++;
				if(countdown<=0){
					System.out.print("#");
					countdown = (size/steppct);
					Runtime.getRuntime().gc();
				}
			}
			System.out.println("  100%");
			Log.getInstance().writeToLog("Free memory after trimming friend lists: "+Runtime.getRuntime().freeMemory(), Log.DEBUG);
			Log.getInstance().writeToLog("Social networks checked:   "+
					"Highest number of friends: "+highest+
					", Lowest number of friends: "+lowest, Log.DEBUG);

			String logstring = "Social network statistics: \n";
			for(int i=0;i<nofFriendsList.length;i++){
				logstring = logstring +"             "+ (i+min)+" friends: "+nofFriendsList[i]+" consumers \n";
			}
			nofFriendsList = null;
			Runtime.getRuntime().gc();
			Log.getInstance().writeToLog(logstring, Log.INFO);
			System.out.println("Social network statistics are written to logfile");
		} catch (final NumberFormatException e) {
			Log.getInstance().writeToLog("The social network could not be analyzed!", Log.WARN);
		} catch (final PropertyNotFoundException e) {
			Log.getInstance().writeToLog("The social network could not be analyzed! ", Log.WARN);
			Log.getInstance().writeToLog(e.getMessage(), Log.WARN);
		}
	}


	/**
	 * Method to generate the social networks between consumers.
	 */
	protected void createSocialConnections(){
		double clusteringFactor = 0.5;
		try {
			clusteringFactor = Double.parseDouble(Config.getInstance().getProperty("ecosim", "Consumers.ClusteringFactor"));
		} catch (final NumberFormatException e1) {
			Log.getInstance().writeToLog("Wrong value of config property: Consumers.ClusteringFactor   Should be a double", Log.WARN);
		} catch (final PropertyNotFoundException e1) {
			Log.getInstance().writeToLog("Could not find property in config file: Consumers.ClusteringFactor", Log.WARN);		}

		final int size = market.getNofConsumers();
		System.out.println("Creating social connections for "+size+" consumers.");
		System.out.println("Starting...");
		int steppct = 50;
		if(size<100) {
			steppct = 20;
		}
		int countdown = (size/steppct);
		for(int i=0;i<(size-1);i++){
			countdown--;
			final Consumer c = market.getConsumer(i);
			boolean hasEnoughFriends = c.hasEnoughFriends();
			final int maxtries = size-i;
			int counter = 0;
			final ArrayList<Integer> lastFriends = new ArrayList<Integer>();
			while(!hasEnoughFriends){
				if(counter==maxtries) {
					break;
				}
				counter++;
				final int friend = (int)(Math.random()*(size-i))+i;
				if(friend!=i&&!c.isFriend(friend)){
					if(c.becomeFriend(friend)){
						market.getConsumer(friend).becomeFriend(i);
						//Tweak for getting more clusters!
						for(int j=0;j<lastFriends.size();j++){
							if(Math.random()<clusteringFactor){
								if(!market.getConsumer(friend).hasEnoughFriends()&&!market.getConsumer(lastFriends.get(j)).hasEnoughFriends()&&!market.getConsumer(friend).isFriend(lastFriends.get(j))){
									market.getConsumer(friend).becomeFriend(lastFriends.get(j));
									market.getConsumer(lastFriends.get(j)).becomeFriend(friend);
								}
							}
						}	
						lastFriends.add(friend);
					}
					else{
						hasEnoughFriends = true;
					}
				}
			}
			if(countdown<=0){
				System.out.print("#");
				countdown = (size/steppct);
			}
		}
		System.out.println("  100%");
		System.out.println("Social networks constructed");
		System.out.println("");

		try {
			final boolean analyzeSocialNetworks = Boolean.parseBoolean(Config.getInstance().getProperty("ecosim", "Debugging.AnalyzeSocialNetworks"));
			if(analyzeSocialNetworks){
				System.out.println("Social network analyzis are: ON");
				analyzeSocialNetworks();
				analyzeSmallWorldEffect();
				final ClusterFinder cf = new ClusterFinder(market);
				cf.findClusters();
				Runtime.getRuntime().gc();
			}
			else{
				System.out.println("Social network analyzis are: OFF");
			}
		} catch (final PropertyNotFoundException e) {
			Log.getInstance().writeToLog("Could not find property in config: Debugging.AnalyzeSocialNetworks", Log.WARN);
		}
	}



	/**
	 * Method to produce a consumer subscribing to a specified service provider.
	 * 
	 * @param spro
	 *            the service provider
	 * 
	 * @return the produced consumer
	 */
	protected Consumer produceBoundConsumer(final ServiceProvider spro){
		final Consumer thisConsumer = new Consumer(IDcounter, spro, market);
		IDcounter++;
		return thisConsumer;
	}


	/**
	 * Method to produce a consumer which is not bound to any service provider.
	 * 
	 * @return the consumer
	 */
	protected Consumer produceSimpleConsumer(){
		final Consumer thisConsumer = new Consumer(IDcounter, market);
		IDcounter++;
		return thisConsumer;
	}

	/**
	 * Method to reset the counter. Must be used when reinitializing the
	 * simulator
	 */
	protected void resetCounter(){
		IDcounter = 0;
	}





}
