/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.environment;

import edu.ntnu.ecosim.agents.consumers.Consumer;
import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.basicfunctionality.Config;
import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.exceptions.NoSuchStrategyException;
import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import edu.ntnu.ecosim.management.SimulationManager;
import java.util.ArrayList;

/**
 * The Market is where consumers and service providers operate. The market holds lists of all actors that are added, and some functionality for activating the agents.
 */
public class Market{

	/**The clock governing the market */
	private Clock clock;

	/**The consumer factory producing the consumers added to the market */
	private ConsumerFactory consfac;

	/** The consumers in the market. */
	private ArrayList<Consumer> consumers;

	/** The service providers in the market. */
	private ArrayList<ServiceProvider> serviceproviders;

	/** The simulation manager. */
	private SimulationManager sm;

	/**
	 * Boolean telling whether or not the market is stopped, 
	 * either because the simlation is finished or because the  
	 * external interface has chosen to stop it.	
	 */
	private boolean stopped;

	/**
	 * The interest rate of the market
	 */
	private double interestRate;
	
	/**
	 * The total time to run for the simulation
	 */
	private int timeToRun;
	
	/** The speed the simulation should be run at */
	private int simSpeed;
	
	/** The planning horizon (months) for the consumers in this market	 */
	private int consumerPlanningHorizon;

	/** Variable used for memory management purposes */
	private long lowestMemory;

	/** Whether or not the market is currently in an activation loop for agents */
	private boolean isInLoop;

	/**
	 * The Market constructor.
	 */
	public Market(final SimulationManager sm){
		this.sm = sm;
		lowestMemory = Runtime.getRuntime().maxMemory();
		consumerPlanningHorizon = 12;
		timeToRun = 0;
		interestRate = 0;
		isInLoop = false;
		try{
			final double ir = Integer.parseInt(Config.getInstance().getProperty("ecosim", "Market.InterestRate"));
			interestRate = (Math.pow(1+(ir/100), (1.0/12.0))-1);
		}
		catch(final PropertyNotFoundException e){
			Log.getInstance().writeToLog("Could not find interest rate in config file", Log.ERROR);
			interestRate = (Math.pow(1.05, (1.0/12.0))-1);
		}
		try{
			consumerPlanningHorizon = Integer.parseInt(Config.getInstance().getProperty("ecosim", "Consumers.PlanningHorizon"))+0;
		}
		catch(final Exception e){
			Log.getInstance().writeToLog("Could not find correct planning horizon in config... Setting it to 12 months", Log.ERROR);
		}
		clock = new Clock(this);
		consfac = new ConsumerFactory(this);
		consumers  = new ArrayList<Consumer>();
		serviceproviders  = new ArrayList<ServiceProvider>();
		stopped = false;
	}


	/**
	 * Method to activate all consumer agents
	 * @throws OutOfMemoryError if memory consumption becomes to large
	 */
	private void activateConsumerAgents() throws OutOfMemoryError{
		final ArrayList<Consumer> tempconsumerlist = new ArrayList<Consumer>(consumers);
		final int nofConsumers = consumers.size();
		Consumer[] tempConsList = new Consumer[consumers.size()];
		for(int i=0;i<nofConsumers;i++){
			if(!stopped){
				isInLoop = true;
				final int rand = (int)(Math.random()*tempconsumerlist.size());
				final Consumer theConsumer = tempconsumerlist.remove(rand);
				theConsumer.performAction();
				tempConsList[theConsumer.getUniqueID()] = theConsumer;
				isInLoop = false;
			}
			else{
				break;}
		}
		if(stopped){
			return;
		}
		consumers = null;
		consumers = new ArrayList<Consumer>();
		for (Consumer element : tempConsList) {
			consumers.add(element);
		}	
		final Runtime r = Runtime.getRuntime();
		tempConsList = null;
		r.gc();
	}

	/**
	 * Method to activate all service provider agents
	 * @throws OutOfMemoryError if memory consumption becomes to large
	 */
	private void activateServiceProviderAgents() throws OutOfMemoryError{
		for(int i=0;i<serviceproviders.size();i++){
			serviceproviders.get(i).performAction();
		}
	}

	/**
	 * Adds one consumer to the market.
	 */
	public void addConsumer(){
		consumers.add(consfac.produceSimpleConsumer());
	}

	/**
	 * Adds one bound consumer to the market
	 * @param sp the service provider the consumer is bound to
	 */
	public void addConsumer(final ServiceProvider sp){
		consumers.add(consfac.produceBoundConsumer(sp));
	}

	/**
	 * Adds the initial (unbound) consumers.
	 * 
	 * @param nofConsumers The number of consumers to add
	 */
	public void addInitialConsumers(final int nofConsumers){
		final Runtime r = Runtime.getRuntime();
		int count = 0;
		System.out.println("Adding unbound consumers...");
		int count2 = 0;
		for(int i=0;i<nofConsumers;i++){
			this.addConsumer();
			count++;
			if(count>=1000){
				count2++;
				System.out.print(".");
				if(count2>50){
					System.out.print("\n");
					count2=0;
				}
				final long freeMb = r.freeMemory()/(1024*1024);
				final long maxMb = r.maxMemory()/(1024*1024);
				if(freeMb<=0.05*maxMb&&freeMb<lowestMemory){
					System.out.println("WARNING: exessive memory usage, to many unbound consumers...");
					System.out.println("Free memory: "+freeMb+"MB out of "+maxMb+"MB");
					Log.getInstance().writeToLog("WARNING: exessive memory usage, to many unbound consumers...", Log.WARN);
					Log.getInstance().writeToLog("Free memory: "+freeMb+"MB out of "+maxMb+"MB", Log.WARN);
					r.gc();
					lowestMemory = freeMb;
				}
				count = 0;
			}
		}
		System.out.print("\n");
	}

	/**
	 * Method to add one service provider to the market
	 * @param name The name of the service provider
	 * @param initsub The number of initial subscribers
	 * @param subFee The subscription fee for this provider
	 * @param bindFee The binding fee for the provider
	 * @param strategy The strategy (innovative, reactive, innoreactive, etc.) for this provider
	 * @param innostr The innovative strength (between 0 and 10)
	 * @param innotype The innovation type this company develops (between 0 and 10, where 0 is "only value-adding" and 10 is "only network effect adding")
	 * @param ecofl The economic flexibility of this company (between 0 and 10)
	 * @throws NoSuchStrategyException If the specified strategy could not be found. 
	 */
	public void addServiceProvider(final String name, final int initsub, final double subFee, final double bindFee, final String strategy, final int innostr, final int innotype, final int ecofl) throws NoSuchStrategyException{
		final ServiceProvider sp = new ServiceProvider(name, initsub, subFee, bindFee, this, strategy, innostr, innotype, ecofl);
		serviceproviders.add(sp);
	}

	/**
	 * Method to get a consumer from the list of consumers
	 * 
	 * @param uniqueID
	 *            the unique ID of the consumer
	 * 
	 * @return the consumer
	 */
	public Consumer getConsumer(final int uniqueID){
		return consumers.get(uniqueID);
	}
	
	/**
	 * Method to get the planning horizon of consumers
	 * @return      the consumerPlanningHorizon
	 */
	public int getConsumerPlanningHorizon(){
		return consumerPlanningHorizon;
	}



	/**
	 * Method to get the interest rate in the market
	 * @return      the interest rate
	 */
	public double getInterestRate(){
		return interestRate;
	}


	/**
	 * Gets the total number of consumers in the market
	 * @return the number of consumers
	 */
	public int getNofConsumers(){
		return consumers.size();
	}



	/**
	 * Gets the number of service providers in the market. 
	 * 
	 * @return The number of service providers in the market
	 */
	public int getNofProviders(){
		return serviceproviders.size();
	}


	/**
	 * Gets the service provider according to the location in the list
	 * 
	 * @param no
	 *            the location in the list
	 * 
	 * @return the service provider
	 */
	public ServiceProvider getServiceProvider(final int no){
		return serviceproviders.get(no);
	}


	/**
	 * Method to get the simulation speed
	 * @return the sleep time (milliseconds) of the main thread
	 */
	public int getSimulationSpeed(){
		return simSpeed;
	}

	/**
	 * Gets the time of the clock (the number of months the simulation has run)
	 * 
	 * @return the time
	 */
	public int getTime() {
		return clock.getTime();
	}


	/**
	 * Gets the time left of the simulation.
	 * 
	 * @return the time left
	 */
	public int getTimeLeft(){
		return timeToRun-getTime();
	}

	/**
	 * Method to get the total number of time steps for this simulation
	 * @return      the number of time steps
	 */
	public int getTimeToRun(){
		return timeToRun;
	}


	/**
	 * Gets the number of unbound consumers in the market.
	 * 
	 * @return the number of unbound consumers
	 */
	public int getUnboundConsumers(){
		int bound= 0;
		for(int i=0;i<serviceproviders.size();i++){
			bound+=serviceproviders.get(i).getNofSubscribers();
		}
		return consumers.size()-bound;
	}



	/**
	 * Gets whether or not the market i stopped
	 * @return    the stopped
	 */
	public boolean isStopped() {
		return stopped;
	}

	
	/**
	 * This method is called for every timestep. 
	 * The method activates all agents in the market. 
	 */
	public void newTimeStep() {
		try{
			activateServiceProviderAgents();
			activateConsumerAgents();
			if(!stopped){
				sm.sendTemporaryResultsXML(""+getTime());
				final Runtime r = Runtime.getRuntime();
				Log.getInstance().writeToLog("Free memory at time "+getTime()+": "+r.freeMemory(),Log.DEBUG);
			}
		}
		catch(final java.lang.OutOfMemoryError e){
			sm.sendErrorMessage("The simulator went out of memory.");
			Log.getInstance().writeToLog(Market.class+"WENT OUT OF MEMORY!!!!!", Log.FATAL);
			clock.stopClock();
		}
	}

	/**
	 * Re-initialize the market.
	 * This method resets all crucial parameters and prepares the market for the next simulation
	 */
	public void reinitMarket(){
		clock.stopClock();
		consfac.resetCounter();
		while(isInLoop){
			try {
				Thread.sleep(50);
			} catch (final InterruptedException e) {
				Log.getInstance().writeToLog(e.toString(), Log.WARN);
			}
		}
		consumers = null;
		serviceproviders = null;
	}

	/**
	 * Method to set the simulation speed
	 * @param ssp the "sleep time" of the main thread
	 */
	public void setSimulationSpeed(final int ssp){
		simSpeed = ssp;
	}



	/**
	 * Method to set the number of intervals the simulator should run (time steps)
	 * @param ttr      the number of time steps that should be run
	 */
	public void setTimeToRun(final int ttr){
		timeToRun = ttr;
	}


	/**
	 * Method to be called when the simulation is done
	 */
	public void simulationDone() {
		sm.sendFinalResultsXML();
	}


	/**
	 * Starts the simulation of the market.
	 */
	public void startMarket(){
		consfac.createSocialConnections();
		clock.start();
		stopped = false;
	}


	/**
	 * Stops the simulation of the market.
	 */
	public void stopMarket(){
		stopped = true;
		clock.stopClock();
	}


}
