/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.management;

import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.environment.Market;
import java.util.Observable;
import org.w3c.dom.Document;

/**
 * The Class SimulationManager. This is a controll class which binds the environment layer and the communication interface layer. The simulation manager is responsible for setting up the simulation, and for sending temporary- and final results.
 */
public class SimulationManager extends Observable implements Runnable{

	/** The market to perform the simulation on. */
	private Market market;

	/** The xml handler managing the xml reading. */
	private XMLHandler xmlhandler;

	/** The received setup document. */
	private Document setupDocument;

	/**
	 * The Constructor.
	 */
	public SimulationManager(){
		setupDocument = null;
		market = new Market(this);
		xmlhandler = new XMLHandler(market, this);
	}


	/**
	 * Method to reset the simulation.
	 */
	public void reset(){
		if(market!=null){
			market.reinitMarket();
		}
	}


	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		if(setupDocument!=null){
			final Document response = xmlhandler.receiveSetup(setupDocument);
			sendXMLDocument(response);	
		}
	}

	/**
	 * Send error message to the application using the simulator
	 * 
	 * @param msg
	 *            the message to send
	 */
	public void sendErrorMessage(final String msg){
		final Document error = xmlhandler.createErrorMessage(msg);
		sendXMLDocument(error);
	}


	/**
	 * Send final results as a XML document.
	 */
	public void sendFinalResultsXML() {
		final Document d = xmlhandler.createFinalResults("Simulation is finished, no error messages..");
		sendXMLDocument(d);
		Log.getInstance().writeToLog("SIMULATION FINISHED", Log.INFO);
	}

	/**
	 * Send temporary results as XML document.
	 * 
	 * @param time
	 *            the current time of the simulation
	 */
	public void sendTemporaryResultsXML(final String time){
		final Document resp = xmlhandler.createTempResults(time);
		if(resp!=null){
			sendXMLDocument(resp);
		}
	}
	
	/**
	 * Send information message as XML document
	 * @param info the information to send
	 */
	public void sendInfoMessage(final String info){
		final Document infomsg = xmlhandler.generateStatusResponse("info", info);
		if(infomsg!=null){
			sendXMLDocument(infomsg);
		}
	}

	/**
	 * Send XML document to the "result receiver" implementation.
	 * 
	 * @param doc
	 *            the doc
	 */
	private void sendXMLDocument(final Document doc){
		setChanged();
		notifyObservers(doc);
	}


	/**
	 * Setup the environment.
	 * 
	 * @param setup
	 *            the document containing information about the simulation
	 */
	public void setupEnvironment(final Document setup) {
		setupDocument = setup;
	}




	/**
	 * Start the simulation.
	 */
	public void startSimulation(){
		Log.getInstance().writeToLog("SIMULATION STARTING", Log.INFO);
		market.startMarket();
	}


	/**
	 * Stop the simulation.
	 */
	public void stopSimulation(){
		market.stopMarket();
	}

}
