/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.management;

import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.environment.Market;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This class is responsible for gathering statistics and (if necessary)
 * use this information to create new statistics.
 */
public class Statistics {

	
	/** The market to operate in. */
	private Market market;
	
	/** The total number of consumers changing provider*/
	private int totalChanges;
	
	/**
	 * The Constructor.
	 */
	protected Statistics(final Market m){
		market = m;
		totalChanges = 0;
	}
	
	/**
	 * Gets the ARPU (Average Revenue Per User).
	 * 
	 * @param provider
	 *            the providers place in the markets service provider list
	 * 
	 * @return the ARPU
	 */
	protected double getARPU(final int provider) {
		return market.getServiceProvider(provider).calculateArpu();
	}
	
	/**
	 * Gets the average consumer expense in the market
	 * 
	 * @return the average consumer expense
	 */
	protected double getAverageConsumerExpense(){
		final int nofcons = market.getNofConsumers();
		int totalrevround = 0;
		for(int i=0;i<market.getNofProviders();i++){
			totalrevround+=market.getServiceProvider(i).getRoundProfit();
		}
		return ((double)totalrevround/nofcons);
	}
	

	/**
	 * Gets the customer churn rate of a service provider.
	 * 
	 * @param provider
	 *            the provider
	 * 
	 * @return the customer churn rate
	 */
	protected double getChurnRate(final int provider){
		return market.getServiceProvider(provider).calculateChurnRate();
	}
	
	/**
	 * Gets the highest value of this round.
	 * 
	 * @param place
	 *            the place in the markets service provider list where the provider is located
	 * 
	 * @return the highest value this round
	 */
	protected double getHighestThisRound(final int place){
		double high = 0;
		for(int i=0;i<market.getNofConsumers();i++){
			final double value = market.getConsumer(i).getCalculatedValue(place);
			if(value>high) {
				high=value;
			}
		}
		return high;
	}
	
	
	
	/**
	 * Gets the number of innovations produced by a specified provider
	 * @param prov the providers place in the market list
	 * @return the number of innovations
	 */
	protected int getInnovationsForProvider(final int prov){
		return market.getServiceProvider(prov).getNofInnovations();
	}

	/**
	 * Gets the total number of changes.
	 * 
	 * @return the nof changes
	 */
	protected int getNofChanges(){
		int counter = 0;
		for(int i=0;i<market.getNofConsumers();i++){
			if(market.getConsumer(i).providerChanged()) {
				counter++;
			}
		}
		totalChanges += counter;
		return totalChanges;
	}
	
	
	/**
	 * Method to get the statistics on produced innovations
	 * @return A string describing the knowledge of innovations
	 */
	protected String getInnovationStats() {
		final Hashtable<String, String> innovations = new Hashtable<String, String>();
		for(int i=0;i<market.getNofProviders();i++){
			final ServiceProvider sp = market.getServiceProvider(i);
			for(int j=0;j<sp.getNofInnovations();j++){
				innovations.put(sp.getInnovation(j).getUniqueID(), sp.getName());
			}
		}
		final int[] knowledge = new int[innovations.size()];
		int counter = 0;
		final Enumeration<String> innos = innovations.keys();
		final int totalCons = market.getNofConsumers();
		String retString = "Innovation statistics: \n";
		while(innos.hasMoreElements()){
			final String innoStr = innos.nextElement();
			knowledge[counter]=0;
			for(int j=0;j<market.getNofConsumers();j++){
				if(market.getConsumer(j).knowsInnovation(innoStr)){
					knowledge[counter]++;
				}
			}
			final int pct = (int)(100*((double)knowledge[counter]/totalCons));
			retString+="Innovation "+innoStr+" by "+innovations.get(innoStr)+" was known by "+pct+"% of the consumers \n";
			counter++;
		}
		return retString;
	}
	
	
}
