/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.management;

import edu.ntnu.ecosim.agents.serviceproviders.ServiceProvider;
import edu.ntnu.ecosim.basicfunctionality.Log;
import edu.ntnu.ecosim.environment.Market;
import edu.ntnu.ecosim.exceptions.InitializationException;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * The Class XMLHandler is responsible for parsing and generating the XML Documents that are used as communication format between the simulator and surrounding framework.
 */
public class XMLHandler {

	/** The market to perform actions upon. */
	private Market market;
	
	/** The statistics to get information from. */
	private Statistics stats;
	
	/** The simulation manager owning this XMLHandler */
	private SimulationManager sm;
	
	/**
	 * The Constructor.
	 * 
	 * @param m
	 *            the m
	 */
	protected XMLHandler(final Market m, final SimulationManager sm){
		this.sm = sm;
		market = m;
		stats = new Statistics(market);
	}

	
	/**
	 * Adds service providers specified in a NodeList to the market.
	 * 
	 * @param providers
	 *            a nodelist containing the providers
	 * 
	 * @return true, if everything went perfect
	 * 
	 * @throws InitializationException
	 *             if there are errors in the setup document
	 */
	private void addServiceProviders(final NodeList providers) throws InitializationException{
		try{
			for(int i=0;i<providers.getLength();i++){
				final String name = getSimpleElementText((Element)providers.item(i),"name");
				final int subscribers = Integer.parseInt(getSimpleElementText((Element)providers.item(i),"subscribers"));
				final double bindfee = Double.parseDouble(getSimpleElementText((Element)providers.item(i),"bindfee"));
				final double subscost = Double.parseDouble(getSimpleElementText((Element)providers.item(i),"subscost"));
				final String strategy = getSimpleElementText((Element)providers.item(i),"strategy");
				final int innostr = Integer.parseInt(getSimpleElementText((Element)providers.item(i),"innostrength"));
				final int ecoflex = Integer.parseInt(getSimpleElementText((Element)providers.item(i),"ecoflex"));
				final int innotype = Integer.parseInt(getSimpleElementText((Element)providers.item(i),"innotype"));
				market.addServiceProvider(name, subscribers, subscost,bindfee , strategy,innostr, innotype, ecoflex);
				Log.getInstance().writeToLog("Added service provider: "+name, Log.INFO);
				sm.sendInfoMessage("Service provider ''"+name+"'' created with "+subscribers+" subscribers.");
			}	
		}
		catch(final Exception e){
			e.printStackTrace();
			throw new InitializationException("Problems with service provider parameters...", e);
		}
		catch(final Error e){
			throw new InitializationException("Error in simulator"+e.getMessage());
		}
	}

	/**
	 * Method to create an error message with the specified content in the
	 * necessary format.
	 * 
	 * @param msg
	 *            The message to include
	 * 
	 * @return A document ready to send to the result receiver
	 */
	protected Document createErrorMessage(final String msg) {
		Document doc = null;
		try {
			final DocumentBuilderFactory dFactory =
				DocumentBuilderFactory.newInstance();
			doc = dFactory.newDocumentBuilder().newDocument();
			final Element root = doc.createElement("response");
			final Element type = doc.createElement("type");
			type.setTextContent("status");
			final Element status = doc.createElement("status");
			status.setTextContent("error");
			final Element info = doc.createElement("information");
			info.setTextContent(msg);
			root.appendChild(type);
			root.appendChild(status);
			root.appendChild(info);
			doc.appendChild(root);
		} catch (final ParserConfigurationException e) {
			Log.getInstance().writeToLog(e.getMessage(), Log.WARN);
		}
		return doc;
	}


	/**
	 * Method to generate the final results to send to the result receiver.
	 * 
	 * @param information
	 *            Information to add to the final results message
	 * 
	 * @return A document ready to send to the result receiver
	 */
	protected Document createFinalResults(final String information){
		Document doc = null;
		String innovationStats = stats.getInnovationStats();
		try {
			final DocumentBuilderFactory dFactory =
				DocumentBuilderFactory.newInstance();
			doc = dFactory.newDocumentBuilder().newDocument();
			final Element root = doc.createElement("response");
			final Element type = doc.createElement("type");
			type.setTextContent("status");
			final Element thestatus = doc.createElement("status");
			thestatus.setTextContent("simulation_finished");
			final Element info = doc.createElement("information");
			if(information!=null){
				info.setTextContent(information+"\n"+innovationStats);
			}
			else{
				info.setTextContent(innovationStats);
			}
			Log.getInstance().writeToLog("Innovation statistics: \n"+innovationStats, Log.INFO);
			root.appendChild(type);
			root.appendChild(thestatus);
			root.appendChild(info);
			doc.appendChild(root);
		} catch (final ParserConfigurationException e) {
			Log.getInstance().writeToLog(e.getMessage()+"\n"+e.getCause(), Log.WARN);
		}
		return doc;
	}

	/**
	 * Creates the temporary results document.
	 * 
	 * @param time
	 *            the time at which the document is created
	 * 
	 * @return the document to send
	 */
	protected Document createTempResults(final String time){
		if(market.isStopped()){
			return null;
		}
		Document doc = null;
		try {
			final DocumentBuilderFactory dFactory =
				DocumentBuilderFactory.newInstance();
			doc = dFactory.newDocumentBuilder().newDocument();
			final Element root = doc.createElement("response");
			final Element type = doc.createElement("type");
			type.setTextContent("temp_results");
			root.appendChild(type);
			final Element thetime = doc.createElement("time");
			thetime.setTextContent(time);
			root.appendChild(thetime);
			final Element totalcons = doc.createElement("totalconsumers");
			totalcons.setTextContent(""+market.getNofConsumers());
			root.appendChild(totalcons);
			final Element totaltrans = doc.createElement("totaltransitions");
			totaltrans.setTextContent(""+stats.getNofChanges());
			root.appendChild(totaltrans);
			final Element avgconsexp = doc.createElement("avgconsumerexpense");
			avgconsexp.setTextContent(""+stats.getAverageConsumerExpense());
			root.appendChild(avgconsexp);
			final int nofSp = market.getNofProviders();
			int totalBoundCustomers = 0;
			for(int i=0;i<nofSp;i++){
				final Element provider = doc.createElement("provider");
				final ServiceProvider sp = market.getServiceProvider(i);
				totalBoundCustomers+=sp.getNofSubscribers();
				//The name of the provider
				final Element name = doc.createElement("name");
				name.setTextContent(sp.getName());
				provider.appendChild(name);
				//the number of customers
				final Element cust = doc.createElement("customers");
				cust.setTextContent(""+sp.getNofSubscribers());
				provider.appendChild(cust);
				//The binding fee cost this round
				final Element cost = doc.createElement("cost");
				cost.setTextContent(""+sp.getBindingFee());
				provider.appendChild(cost);
				//The subscription fee this round
				final Element subf = doc.createElement("subfee");
				subf.setTextContent(""+sp.getSubscriptionFee());
				provider.appendChild(subf);
				//the highest value this round
				final Element high = doc.createElement("highestvalue");
				high.setTextContent(""+stats.getHighestThisRound(i));
				provider.appendChild(high);
				//the revenue this round
				final Element revenue = doc.createElement("totalrevenue");
				revenue.setTextContent(""+sp.getProfit());
				provider.appendChild(revenue);
				final Element arpu = doc.createElement("arpu");
				arpu.setTextContent(""+stats.getARPU(i));
				provider.appendChild(arpu);
				final Element churn = doc.createElement("churnrate");
				churn.setTextContent(""+stats.getChurnRate(i));
				provider.appendChild(churn);
				final Element innovations = doc.createElement("innovations");
				innovations.setTextContent(""+stats.getInnovationsForProvider(i));
				provider.appendChild(innovations);
				root.appendChild(provider);
			}
			final Element unbound = doc.createElement("unboundconsumers");
			final int unb = market.getNofConsumers() - totalBoundCustomers;
			unbound.setTextContent(""+unb);
			root.appendChild(unbound);
			doc.appendChild(root);
		} catch (final ParserConfigurationException e) {
			Log.getInstance().writeToLog(e.getMessage()+"\n"+e.getCause(), Log.WARN);
		}
		return doc;		
	}


	/**
	 * Generate status response message.
	 * 
	 * @param information
	 *            the extra information to add, describing the status
	 * @param status
	 *            the status of the simulator
	 * 
	 * @return the document containing the information of the response
	 */
	protected Document generateStatusResponse(final String status, final String information){
		Document doc = null;
		try {
			final DocumentBuilderFactory dFactory =
				DocumentBuilderFactory.newInstance();
			doc = dFactory.newDocumentBuilder().newDocument();
			final Element root = doc.createElement("response");
			final Element type = doc.createElement("type");
			type.setTextContent("status");
			final Element thestatus = doc.createElement("status");
			thestatus.setTextContent(status);
			final Element info = doc.createElement("information");
			info.setTextContent(information);
			root.appendChild(type);
			root.appendChild(thestatus);
			root.appendChild(info);
			doc.appendChild(root);
		} catch (final ParserConfigurationException e) {
			Log.getInstance().writeToLog(e.getMessage()+"\n"+e.getCause(), Log.WARN);
		}
		return doc;
	}

	/**
	 * (Help method) 
	 * Gets the first element that fits the provided name.
	 * 
	 * @param element
	 *            the element to fetch the new element from
	 * @param name
	 *            the name of the element to get
	 * 
	 * @return the first element fitting the name
	 * 
	 * @throws Exception
	 *             throws an exception if the text can not be found
	 */
	private Element getFirstElement(final Element element, final String name) throws Exception {
		final NodeList nl = element.getElementsByTagName(name);
		if (nl.getLength() < 1) {
			throw new RuntimeException("Element: " + element
					+ " does not contain: " + name);
		}
		return (Element) nl.item(0);
	}

	/**
	 * (Help method) 
	 * Gets the "simple element"s text content.
	 * 
	 * @param node
	 *            the node to get the text from
	 * 
	 * @return the text of the element
	 * 
	 * @throws Exception
	 *             throws an exception if the text can not be found
	 */
	private String getSimpleElementText(final Element node) throws Exception{
		final StringBuffer sb = new StringBuffer();
		final NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			final Node child = children.item(i);
			if (child instanceof Text) {
				sb.append(child.getNodeValue());
			}
		}
		return sb.toString();
	}




	/**
	 * (Help method) 
	 * Gets the text of the element with the specified name.
	 * 
	 * @param node
	 *            the node to fetch the element from
	 * @param name
	 *            the name of the element
	 * 
	 * @return the text of the element
	 * 
	 * @throws Exception
	 *             throws an exception if the text can not be found
	 */
	private String getSimpleElementText(final Element node, final String name) throws Exception{
		final Element namedElement = getFirstElement(node, name);
		return getSimpleElementText(namedElement);
	}



	/**
	 * Receive the setup document.
	 * 
	 * @param setupdoc
	 *            the document containing setup information
	 * 
	 * @return A document conaining information about the status of the
	 *         simulator after setup
	 */
	protected Document receiveSetup(final Document setupdoc){
		try{
			final Element setup = setupdoc.getDocumentElement();
			final NodeList providers = setup.getElementsByTagName("provider");
			final NodeList environment = setup.getElementsByTagName("environment");
			addServiceProviders(providers);
			setupEnvironment((Element)environment.item(0));
		}
		catch(final Exception e){
			e.printStackTrace();
			return generateStatusResponse("setup_failed", "Something wrong with the parameters of the service provider: "+e.getMessage());
		}
		return generateStatusResponse("setup_ok", "Initialization of simulator went perfect");
	}

	/**
	 * Setup the environment of the simulation.
	 * 
	 * @param environment
	 *            an Element conaining information about the environment
	 * 
	 * @throws InitializationException
	 *            thrown if the initialization does not perform as expected
	 */
	private void setupEnvironment(final Element environment) throws InitializationException{
		try{
			market.setSimulationSpeed(Integer.parseInt(getSimpleElementText(environment,"simulationspeed")));
			market.setTimeToRun(Integer.parseInt(getSimpleElementText(environment,"simulationlength")));
			Integer nofCons = Integer.parseInt(getSimpleElementText(environment,"unboundpopulation"));
			market.addInitialConsumers(nofCons);
			sm.sendInfoMessage("Market initialized with "+nofCons+" unbound consumers");
		}
		catch(final Exception e){
			throw new InitializationException("Problems with environment variables", e);
		}
		catch(final Error e){
			throw new InitializationException("Error in simulator: "+e.getMessage()+", "+e.getCause());
		}
	}


}
