/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.visual;

import edu.ntnu.ecosim.communication.EcoSim;
import edu.ntnu.ecosim.communication.ResultReceiver;
import edu.ntnu.ecosim.visual.graph.ARPUGraphPanel;
import edu.ntnu.ecosim.visual.graph.ChurnGraphPanel;
import edu.ntnu.ecosim.visual.graph.ConsumerExpensePanel;
import edu.ntnu.ecosim.visual.graph.ConsumerPiePanel;
import edu.ntnu.ecosim.visual.graph.InnovationsPanel;
import edu.ntnu.ecosim.visual.graph.RevenueGraphPanel;
import edu.ntnu.ecosim.visual.graph.SubscriberGraphPanel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

import org.w3c.dom.Document;

/**
 * This is the class containing the main method and creating the main frame.
 * 
 * @author martin.andreas.borke
 */
public class Visual extends JFrame implements ActionListener {

	/** The Constant serialVersionUID used for serialization purposes. */
	private static final long serialVersionUID = 244696024122496431L;

	/**
	 * The main method.
	 * 
	 * @param args
	 *            additional arguments (not used)
	 */
	public static void main(final String[] args) {
		new Visual();
	}

	// Private variables
	private ConfigPanel2 cp2;

	private SimulationPanel sp;

	private EnvironmentPanel2 ep2;

	private XMLManager xmlman;

	private EcoSim es;

	private ResultReceiver rr;

	private SubscriberGraphPanel cgp;

	private ConsumerExpensePanel ccp;

	private ConsumerPiePanel cpp;

	private ARPUGraphPanel agp;

	private RevenueGraphPanel rgp;

	private ChurnGraphPanel chgp;

	private InnovationsPanel ip;

	private JMenuBar menuBar;

	private JMenu menu;

	private JMenu help;

	private JMenuItem restart;

	private JPanel buttonPanel;

	private JButton start;

	private JButton stop;

	private JButton reset;

	private JPanel collectorpanel;

	private JPanel collectorpanel2;

	private JTextField infoArea;

	private JTabbedPane jtp;

	private int totaltime;

	private boolean isRunning;

	private int imageIterator;

	private String imageFolder;

	private int runcounter;

	private int totalruns;

	/**
	 * The Constructor.
	 */
	public Visual() {
		totalruns = 0;
		runcounter = 0;
		imageIterator = 0;
		isRunning = false;
		totaltime = 0;
		initComponents();
		rr = new Receiver(xmlman);
		es = new EcoSim(rr);
		this.setIconImage(createImageIcon("images/eslogo.gif").getImage());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(final ActionEvent arg0) {
		if ("restart".equals(arg0.getActionCommand())) {
			reset();
		} else if ("start".equals(arg0.getActionCommand())) {
			createImageFolder();
			jtp.setSelectedIndex(2);
			prepareForSetup();
			JOptionPane.showMessageDialog(this,
					"The simulator is being initiated.\n\n"
							+ "This can take some minutes depending on\n"
							+ "the number of consumers. \n \nPlease wait.",
					"Important information", JOptionPane.INFORMATION_MESSAGE);
		} else if ("stop".equals(arg0.getActionCommand())) {
			sp.appendText("\n ...........  SIMULATOR STOPPED  ........ \n\n");
			stopSimulator();
			stop.setEnabled(false);
		} else if ("exit".equals(arg0.getActionCommand())) {
			System.exit(0);
		} else if ("about".equals(arg0.getActionCommand())) {
			String info = "This is the master's thesis by Martin Brke at \n"
					+ "NTNU (Norwegian University of Science and Technology), \n"
					+ "developed the spring of 2007. \n\n"
					+ "EcoSim is a simulator for mobile innovation. For further \n"
					+ "information, please consult the thesis report or contact \n"
					+ "the author at: \n\n" + "martinb@stud.ntnu.no \n  or \n"
					+ "martin.andreas.borke@accenture.com";
			JOptionPane.showMessageDialog(this, info, "About",
					JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Adds the global graph points (for the graps not belonging to specific
	 * providers)
	 * 
	 * @param time
	 *            the time
	 * @param avgcost
	 *            the average cost in market
	 */
	public void addGlobalGraphPoints(final double avgcost, final int time) {
		ccp.addPoint(avgcost, time);
	}

	/**
	 * Adds the graph points for a provider
	 * 
	 * @param time
	 *            the time
	 * @param churnrate
	 *            the churn rate
	 * @param provider
	 *            the providers place in the local list
	 * @param innovations
	 *            the number of innovations for this provider
	 * @param totalrevenue
	 *            the total revenue for the provider
	 * @param nofsubs
	 *            the number of subscribers
	 * @param arpu
	 *            the ARPU
	 */
	public void addGraphPoints(final int provider, final int nofsubs,
			final int time, final int totalrevenue, final double arpu,
			final double churnrate, final int innovations) {
		cgp.addPoint(provider, nofsubs, time);
		agp.addPoint(provider, time, arpu);
		rgp.addPoint(provider, time, totalrevenue);
		chgp.addPoint(provider, time, churnrate);
		cpp.setSubscribers(provider, nofsubs, time);
		ip.setNofInnovations(provider, innovations);
	}

	/**
	 * Adds a new provider to the graphs
	 * 
	 * @param nofSubscribers
	 *            the nof subscribers
	 * @param name
	 *            the name
	 * @param subcost
	 *            the subscription cost (initial, used to configure graph sizes)
	 */
	public void addProvider(final int nofSubscribers, final String name,
			final int subcost) {
		cgp.addProvider(nofSubscribers, name);
		agp.addProvider(name, subcost);
		rgp.addProvider(name, subcost);
		chgp.addProvider(name);
		ccp.addProvider(subcost, name);
		cpp.addProvider(nofSubscribers, name);
		ip.addProvider(name);
	}

	/**
	 * This method is used when the two statistics panels are to be written to
	 * JPG files! It also captures the textual output to a .TXT file in the
	 * image folder.
	 */
	private void captureStatistics() {
		try {
			final BufferedWriter out = new BufferedWriter(new FileWriter(
					imageFolder + System.getProperty("file.separator")
							+ "statistics.txt", true));
			out
					.write("########################################################### \n \n");
			out.write("RUN " + imageIterator + ": \n\n");
			out.write(sp.getText());
			out.write("\n");
			out
					.write("###########################################################");
			out.write("\n");
			out.close();
			ScreenImage im = new ScreenImage();
			im.createImage(collectorpanel, imageFolder
					+ System.getProperty("file.separator") + "screenshot"
					+ imageIterator + "a.jpg");
			im.createImage(collectorpanel2, imageFolder
					+ System.getProperty("file.separator") + "screenshot"
					+ imageIterator + "b.jpg");
			imageIterator++;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Check whether the simulation is finished.
	 * 
	 * @param time
	 *            the current received "time" variable
	 * 
	 * @return true, if simulation is finished
	 */
	public boolean checkFinished(final int time) {
		if (time == totaltime) {
			restart.setEnabled(true);
			return true;
		}
		return false;
	}

	/**
	 * Method used to create the folder that will contain the images. The folder
	 * will be given a name reflecting the date and time of creation to ensure
	 * that no two folders conflict
	 * 
	 */
	private void createImageFolder() {
		final Calendar cal = new GregorianCalendar();
		String h = "" + cal.get(Calendar.HOUR_OF_DAY);
		String m = "" + cal.get(Calendar.MINUTE);
		String sec = "" + cal.get(Calendar.SECOND);
		if (Integer.parseInt(h) < 10) {
			h = "0" + h;
		}
		if (Integer.parseInt(m) < 10) {
			m = "0" + m;
		}
		if (Integer.parseInt(sec) < 10) {
			sec = "0" + sec;
		}
		String y = ""+cal.get(Calendar.YEAR);
		String d = ""+cal.get(Calendar.DAY_OF_MONTH);
		String mon = ""+(1+cal.get(Calendar.MONTH));
		if(Integer.parseInt(d)<10){
			d = "0"+d;
		}
		if(Integer.parseInt(mon)<10){
			mon = "0"+mon;
		}
		final String time = "" + h + "" + m + "" + sec;
		final String date = "" + y + "-"+ mon + "-" + d;
		final String filepath = "output/statistics_" + date + "_" + time;
		final File path = new File(filepath);
		if (!path.exists()) {
			path.mkdirs();
		}
		imageFolder = filepath;
	}

	/**
	 * Method to create an ImageIcon from an image Returns an ImageIcon, or null
	 * if the path was invalid.
	 * 
	 * @param path
	 *            the path
	 * 
	 * @return the image icon
	 */
	private ImageIcon createImageIcon(final String path) {
		final java.net.URL imgURL = Visual.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	/**
	 * Creates the setup document from the settings in ConfigPanel and
	 * EnvironmentPanel
	 * 
	 * @return the setup document
	 */
	public Document createSetupDocument() {
		return xmlman.createSetupDocument(cp2.getProviders(), ep2
				.getVariables());
	}

	/**
	 * Gets the environment variables.
	 * 
	 * @return the environment variables
	 */
	public String[] getEnvironmentVariables() {
		return ep2.getVariables();
	}

	/**
	 * Gets the service providers.
	 * 
	 * @return the service providers
	 */
	public ArrayList<String[]> getServiceProviders() {
		return cp2.getProviders();
	}

	/**
	 * Initialize the components.
	 */
	private void initComponents() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Error setting Look and Feel: " + e);
		}
		getContentPane().removeAll();
		setLayout(new BorderLayout(10, 0));
		menuBar = new JMenuBar();
		menu = new JMenu("Simulator");
		menuBar.add(menu);
		help = new JMenu("Help");
		menuBar.add(help);
		restart = new JMenuItem("New simulation...");
		restart.setActionCommand("restart");
		restart.addActionListener(this);
		restart.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
				Event.CTRL_MASK));
		JMenuItem exit = new JMenuItem("Exit");
		exit.setActionCommand("exit");
		exit.addActionListener(this);
		exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
				Event.CTRL_MASK));
		JMenuItem about = new JMenuItem("About");
		about.setActionCommand("about");
		about.addActionListener(this);
		help.add(about);
		menu.add(restart);
		menu.add(exit);
		setJMenuBar(menuBar);

		setTitle("EcoSim v1.0 Beta - visual interface");
		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
		this.setSize(1200, 870);
		this.setMinimumSize(new Dimension(1200, 870));
		setResizable(false);
		restart.setEnabled(false);

		if (runcounter <= 0) {
			cp2 = new ConfigPanel2(this);
			ep2 = new EnvironmentPanel2();
		}

		sp = new SimulationPanel(this);
		cgp = new SubscriberGraphPanel();
		ccp = new ConsumerExpensePanel();
		agp = new ARPUGraphPanel();
		rgp = new RevenueGraphPanel();
		chgp = new ChurnGraphPanel();
		cpp = new ConsumerPiePanel();
		ip = new InnovationsPanel();

		collectorpanel = new JPanel(new GridLayout(2, 2, 10, 10));
		collectorpanel.setBackground(new java.awt.Color(233, 241, 243));
		collectorpanel.add(cgp);
		collectorpanel.add(chgp);
		collectorpanel.add(agp);
		collectorpanel.add(rgp);

		collectorpanel2 = new JPanel(new GridLayout(2, 2, 10, 10));
		collectorpanel2.setBackground(new java.awt.Color(233, 241, 243));
		collectorpanel2.add(ccp);
		collectorpanel2.add(cpp);
		collectorpanel2.add(ip);

		final Container contents = getContentPane();
		jtp = new JTabbedPane();
		jtp.setPreferredSize(new Dimension(1200, 700));
		final ImageIcon icon_sp = createImageIcon("images/config2.gif");
		jtp.addTab("Service Providers", icon_sp, cp2,
				"Configuration of Service Providers");
		final ImageIcon icon_ev = createImageIcon("images/config1.gif");
		jtp.addTab("Environment Variables", icon_ev, ep2,
				"Configuration of Environment Variables");
		final ImageIcon icon_rep = createImageIcon("images/report.gif");
		jtp.addTab("Simulator Response", icon_rep, sp,
				"Textual Response from Simulator");
		final ImageIcon icon_stat = createImageIcon("images/ico_statistics.gif");
		jtp.addTab("Statistics 1", icon_stat, collectorpanel,
				"Graphical Statistics (1 of 2)");
		jtp.addTab("Statistics 2", icon_stat, collectorpanel2,
				"Graphical Statistics (2 of 2)");
		contents.add(jtp, BorderLayout.CENTER);

		jtp.setMnemonicAt(0, KeyEvent.VK_P);
		jtp.setMnemonicAt(1, KeyEvent.VK_E);
		jtp.setMnemonicAt(2, KeyEvent.VK_S);
		jtp.setMnemonicAt(3, KeyEvent.VK_1);
		jtp.setMnemonicAt(4, KeyEvent.VK_2);

		buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 30, 20));
		buttonPanel.setBackground(Color.lightGray);
		start = new JButton("START");
		start.setActionCommand("start");
		start.addActionListener(this);
		start.setEnabled(false);
		stop = new JButton("STOP");
		stop.setActionCommand("stop");
		stop.addActionListener(this);
		stop.setEnabled(false);
		reset = new JButton("RESET");
		reset.setActionCommand("restart");
		reset.addActionListener(this);
		reset.setEnabled(false);
		start
				.setToolTipText("Start the simulation (service providers must be added).");
		stop.setToolTipText("Stop the current simulation.");
		reset.setToolTipText("Reset the simulator.");
		buttonPanel.add(start);
		buttonPanel.add(stop);
		buttonPanel.add(reset);
		infoArea = new JTextField(10);
		infoArea.setEnabled(false);
		infoArea
				.setToolTipText("Which round of the simulations that is currently being simulated");
		buttonPanel.add(infoArea);
		contents.add(buttonPanel, BorderLayout.PAGE_END);
		setVisible(true);
		xmlman = new XMLManager(this, sp);
	}

	/**
	 * Repaint graphs.
	 */
	public void repaintGraphs() {
		cgp.repaint();
		agp.repaint();
		rgp.repaint();
		chgp.repaint();
		ccp.repaint();
		cpp.repaint();
		ip.repaint();
	}

	/**
	 * Method used for reseting all internal settings related to a simulation.
	 * 
	 */
	private void reset() {
		if (start.isEnabled() && runcounter == 0) {
			cp2.clearProviders();
			start.setEnabled(false);
		} else {
			runcounter = 0;
			es.resetSim();
			cp2 = null;
			sp = null;
			ep2 = null;
			xmlman = null;
			totaltime = 0;
			getContentPane().removeAll();
			initComponents();
			xmlman = new XMLManager(this, sp);
			rr = new Receiver(xmlman);
			es = new EcoSim(rr);
			isRunning = false;
			this.repaint();
		}
	}

	/**
	 * Method called when the simulation is finished.
	 */
	public void setFinished() {
		Runtime r = Runtime.getRuntime();
		try {
			// Sleep for 100ms to protect from conflicts
			Thread.sleep(100);
		} catch (InterruptedException e) {
		}
		r.gc();
		captureStatistics();
		restart.setEnabled(true);
		stop.setEnabled(false);
		start.setEnabled(false);
		reset.setEnabled(true);
		runcounter--;
		if (runcounter > 0) {
			totaltime = 0;
			rr = null;
			es = null;
			isRunning = false;
			initComponents();
			xmlman = null;
			xmlman = new XMLManager(this, sp);
			rr = new Receiver(xmlman);
			es = new EcoSim(rr);
			totalruns++;
			infoArea.setText("Run " + totalruns + " out of "
					+ (totalruns + runcounter - 1));
			this.repaint();
			jtp.setSelectedIndex(3);
			prepareForSetup();
		}
	}

	/**
	 * Setup the graphs.
	 */
	public void setupGraph() {
		final int total = cp2.getNofConsumers() + ep2.getNofConsumers();
		cgp.setTotalSubs(total);
		rgp.setTotalSubs(total);
		totaltime = ep2.getTotalTime();
		cgp.setTotalTime(totaltime);
		agp.setTotalTime(totaltime);
		rgp.setTotalTime(totaltime);
		chgp.setTotalTime(totaltime);
		ccp.setTotalTime(totaltime);
	}

	/**
	 * Setup the simulator.
	 */
	public void setupSimulator() {
		reset.setEnabled(false);
		restart.setEnabled(false);
		if (runcounter <= 0) {
			runcounter = ep2.getNumberOfRuns();
			infoArea.setText("Run 1 out of " + runcounter);
			infoArea.repaint();
			es.setupSim(createSetupDocument());
			ep2.freeze();
			cp2.freeze();
			totalruns = 1;
		} else {
			es.setupSim(createSetupDocument());
		}
	}

	/**
	 * Method that may be called when a service provider is added. This will
	 * enable the "Start" button.
	 */
	public void spAdded() {
		if (!isRunning) {
			start.setEnabled(true);
			reset.setEnabled(true);
			restart.setEnabled(true);
			this.repaint();
		}
	}

	/**
	 * Starting the simulation
	 * 
	 * In this step, the setup document is sent to the simulator
	 */
	private void prepareForSetup() {
		start.setEnabled(false);
		start.repaint();
		sp.appendText("Waiting for response from simulator....\n");
		sp.repaint();
		setupGraph();
		setupSimulator();
	}

	/**
	 * Start simulator.
	 * 
	 * In this step, the simulator is actually started
	 */
	public void startSimulator() {
		es.startSim();
		stop.setEnabled(true);
		jtp.setSelectedIndex(3);
	}

	/**
	 * Stop simulator.
	 */
	public void stopSimulator() {
		es.stopSim();
		setFinished();

	}

}
