/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.visual;

import java.io.Serializable;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * The XML manager is the visual interface's xml parsing module. This class creates
 * and reads xml documents. 
 * @author     martin.andreas.borke
 */
public class XMLManager implements Serializable{

	/** The Constant serialVersionUID used for serialization purposes. */
	private static final long serialVersionUID = 1571673062564410818L;
	
	/** The "Visual" object, to access content from other parts of the interface. */
	Visual visual;
	
	/** The panel to send information to*/
	SimulationPanel simpanel;
	
	/**
	 * The Constructor.
	 * 
	 * @param visual
	 *            the visual object (The JFrame)
	 */
	public XMLManager(final Visual visual, final SimulationPanel simpanel){
		this.simpanel = simpanel;
		this.visual = visual;
	}
	
	/**
	 * Creates a setup document from the service provider parameters
	 * and environment parameters
	 * 
	 * @param envParam
	 *            the environment parameters
	 * @param pro
	 *            the service providers
	 * 
	 * @return the setup document
	 */
	public Document createSetupDocument(final ArrayList<String[]> pro, final String[] envParam){
		Document doc = null;
		try {
			final DocumentBuilderFactory dFactory =
				DocumentBuilderFactory.newInstance();
			doc = dFactory.newDocumentBuilder().newDocument();
			final Element root = doc.createElement("setup");
			//ENVIRONMENT
			final Element env = doc.createElement("environment");
			final Element unb = doc.createElement("unboundpopulation");
			final Element simsp = doc.createElement("simulationspeed");
			final Element simlength = doc.createElement("simulationlength");
			unb.setTextContent(envParam[0]);
			simsp.setTextContent(envParam[2]);
			simlength.setTextContent(envParam[1]);
			env.appendChild(unb);
			env.appendChild(simsp);
			env.appendChild(simlength);
			root.appendChild(env);
			//END ENVIRONMENT
			//PROVIDERS
			for(int i=0;i<pro.size();i++){
				final String[] res = pro.get(i);
				final Element prov = doc.createElement("provider");
				final Element name = doc.createElement("name");
				name.setTextContent(res[0]);
				prov.appendChild(name);
				final Element sub = doc.createElement("subscribers");
				sub.setTextContent(res[1]);
				prov.appendChild(sub);
				final Element bindf = doc.createElement("bindfee");
				bindf.setTextContent(res[2]);
				prov.appendChild(bindf);
				final Element suc = doc.createElement("subscost");
				suc.setTextContent(res[3]);
				prov.appendChild(suc);
				final Element strat = doc.createElement("strategy");
				strat.setTextContent(res[4]);
				prov.appendChild(strat);
				final Element ecoflex = doc.createElement("ecoflex");
				ecoflex.setTextContent(res[5]);
				prov.appendChild(ecoflex);
				final Element innostrength = doc.createElement("innostrength");
				innostrength.setTextContent(res[6]);
				prov.appendChild(innostrength);
				final Element innotype = doc.createElement("innotype");
				innotype.setTextContent(res[7]);
				prov.appendChild(innotype);
				root.appendChild(prov);
				visual.addProvider(Integer.parseInt(res[1]),res[0], Integer.parseInt(res[3]));
			}
			//END PROVIDERS
			doc.appendChild(root);
		} catch (final ParserConfigurationException e) {
			e.printStackTrace();
		}
		return doc;
	}
	
	
	
	/**
	 * (Help method)
	 * Gets the first element with the specified name
	 * 
	 * @param element
	 *            the parent element
	 * @param name
	 *            the name
	 * 
	 * @return the first element
	 */
	private Element getFirstElement(final Element element, final String name) {
		final NodeList nl = element.getElementsByTagName(name);
		if (nl.getLength() < 1) {
			throw new RuntimeException("Element: " + element
					+ " does not contain: " + name);
		}
		return (Element) nl.item(0);
	}
	
	/**
	 * (Help method)
	 * Gets the text of the simple element
	 * 
	 * @param node
	 *            the node where the text is located
	 * 
	 * @return the text
	 */
	private String getSimpleElementText(final Element node) {
		final StringBuffer sb = new StringBuffer();
		final NodeList children = node.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			final Node child = children.item(i);
			if (child instanceof Text) {
				sb.append(child.getNodeValue());
			}
		}
		return sb.toString();
	}
	
	/**
	 *(Help method)
	 * Gets the text of the first element with the specified name
	 * 
	 * @param node
	 *            the parent node to start searching from
	 * @param name
	 *            the name of the element
	 * 
	 * @return the text
	 */
	private String getSimpleElementText(final Element node, final String name) {
		final Element namedElement = getFirstElement(node, name);
		return getSimpleElementText(namedElement);
	}
	
	
	public void receiveResponse(final Document doc){
		final Element response = doc.getDocumentElement();
		final String type = getSimpleElementText(getFirstElement(response, "type"));
		if(type.equals("status")){
			receiveStatus(response);
		}
		else if(type.equalsIgnoreCase("temp_results")){
			receiveTempResults(response);
		}
	}
	
	
	/**
	 * Receive status message and display the information.
	 * 
	 * @param response
	 *            the response containing the status message
	 */
	private void receiveStatus(final Element response){
		String toprint = "Response from simulation engine: \n";
		final String status = getSimpleElementText(getFirstElement(response, "status"));  
		toprint+= "Status:      "+status+"\n";
		toprint+= "Information: "+getSimpleElementText(getFirstElement(response, "information"))+"\n";
		if(status.equalsIgnoreCase("setup_ok")){
			simpanel.appendText(toprint);
			simpanel.appendText("\n\n.......  STARTING SIMULATOR .....\n\n");
			visual.startSimulator();
		}
		else if(status.equalsIgnoreCase("setup_failed")){
			simpanel.appendText(toprint);
			simpanel.appendText("\n\n........SETUP FAILED...........\n\n");
		}
		else if(status.equalsIgnoreCase("error")){
			simpanel.appendText(toprint);
			simpanel.appendText("\n There was an error in the simulator.... ");
		}
		else if(status.equalsIgnoreCase("simulation_finished")){
			simpanel.appendText(toprint);
			simpanel.appendText("\n ........ SIMULATION FINISHED! ......... \n");
		}
		else if(status.equalsIgnoreCase("info")){
			simpanel.appendText(""+getSimpleElementText(getFirstElement(response, "information"))+"\n");
		}
		else{
			System.out.println("Received strange status message....");
		}
	}
	
	/**
	 * Receive temporary results and update graphs
	 * 
	 * @param response
	 *            the response from the simulator
	 */
	private void receiveTempResults(final Element response){
		final String time = getSimpleElementText(response, "time");
		final String consumers = getSimpleElementText(response, "totalconsumers");
		final String transitions = getSimpleElementText(response, "totaltransitions");
		final String unbound = getSimpleElementText(response, "unboundconsumers");
		final NodeList providers  = response.getElementsByTagName("provider");
		String print = "";
		print+="..................................................\n\n";
		print+="TIME                          : "+time+"\n";
		print+="Total consumers               : "+consumers+"\n";
		print+="Total transitions             : "+transitions+"\n";
		final double avgtrans = Double.parseDouble(transitions)/Double.parseDouble(consumers);
		print+="Avg. transitions per consumer : "+avgtrans+"\n";
		final Double unb = Double.parseDouble(unbound);
		final Double cons = Double.parseDouble(consumers);
		final double pctdbl = (unb/cons)*100;
		final double pctunbint = (int)pctdbl;
		final double avgconsexp = Double.parseDouble(getSimpleElementText(response,"avgconsumerexpense"));
		print+= "Unbound consumers:"+unbound+" ("+pctunbint +"%)\n";
		visual.addGlobalGraphPoints(avgconsexp, Integer.parseInt(time));
		for(int i=0;i<providers.getLength();i++){
			final Element prov = (Element)providers.item(i);
			final String name = getSimpleElementText(prov,"name");
			final String customers = getSimpleElementText(prov, "customers");
			final double hival = Double.parseDouble(getSimpleElementText(prov, "highestvalue"));
			final String cost = getSimpleElementText(prov, "cost");
			final String subf = getSimpleElementText(prov, "subfee");
			final String totalrevenue = getSimpleElementText(prov, "totalrevenue");
			final int revenue = Integer.parseInt(totalrevenue);
			final double cust = Double.parseDouble(customers);
			final double pct = (cust/cons)*100;
			final int pctint = (int)pct;
			final int hivalue = (int)hival;
			final double arpu = Double.parseDouble(getSimpleElementText(prov, "arpu"));
			final double churnrate = Double.parseDouble(getSimpleElementText(prov, "churnrate"));
			final int innovations = Integer.parseInt(getSimpleElementText(prov, "innovations"));
			print+= name+" : "+customers+" customers ("+pctint+"%) - highest valuation: "+hivalue+" - binding fee: "+cost+" - subscription fee: "+subf+"  - total revenue: "+totalrevenue+"\n";
			if(!time.equals("0")){
				visual.addGraphPoints(i, Integer.parseInt(customers), Integer.parseInt(time), revenue, arpu, churnrate, innovations);
			}
		}
		simpanel.appendText(print);
		visual.repaint();
		final int inttime = Integer.parseInt(time);
		final boolean finished = visual.checkFinished(inttime);
		if(finished){
			visual.setFinished();
		}
		
	}
	
	
}
