/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.visual.graph;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;

/**
 * The ARPU Graph Panel displays information about ARPU (Average Revenue Per User).
 */
public class ARPUGraphPanel extends GraphPanel{
	
	/** The Constant serialVersionUID, used for serializability */
	private static final long serialVersionUID = 1471065330727603915L;
	
	/** The max subscription cost. */
	private int maxSubscriptionCost;
	
	/** The providers. */
	private int providers;
	
	/**
	 * The Constructor of the ARPU Graph panel.
	 */
	public ARPUGraphPanel(){
		this.setBackground(background);
		lines = new ArrayList<GraphLine>();
		providers = 0;
		time = 0;
		totaltime = 0;
		maxSubscriptionCost = 0;
	}
	
	
	
	
	/**
	 * Adds a provider to this graph
	 * 
	 * @param name
	 *            the name of the provider
	 * @param subcost
	 *            the initial subscription cost for this provider
	 */
	public void addProvider(String name, int subcost){
		int colorint = providers%colorlist.length;
		lines.add(new GraphLine(name,colorlist[colorint], 10, graphysize));
		providers++;
		if(subcost>maxSubscriptionCost){maxSubscriptionCost=subcost;}		
	}
	
	/**
	 * Adds a new point to the graph
	 * 
	 * @param time
	 *            the time of the new point
	 * @param provider
	 *            the provider to add the point to
	 * @param arpu
	 *            the calculated ARPU
	 */
	public void addPoint(int provider, int time, double arpu){
		if(this.time!=time){this.time=time;}
		int x = 10 + (int)(time*timestep);
		double maxArp = maxSubscriptionCost*3;
		double pct = arpu/maxArp;
		int y = graphysize-(int)(pct*(graphysize-10));
		lines.get(provider).addPoint(x, y);
	}
	
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics comp){
		super.paintComponent(comp); 
		Graphics2D comp2D = (Graphics2D) comp; 
		comp2D.setColor(background); 
		comp2D.fillRect(0, 0, xsize, ysize); 

		comp2D.setColor(Color.black);
		float yf = Float.parseFloat(graphysize+"F");
		float xf = Float.parseFloat(graphxsize+"F");
		comp2D.draw(new Line2D.Float(10F,yf,xf+10F,yf));
		comp2D.draw(new Line2D.Float(10F,yf,10F,10F));

		comp2D.drawString("Average Revenue Per User (ARPU)", Float.parseFloat(((xsize/2)-60)+"F"), 20F);
		
		comp2D.drawString("TIME: "+time, Float.parseFloat(((xsize/2)-5)+"F"), 35F);
		if(totaltime!=0){
			int stepsize = (int)(((double)totaltime/10)*timestep);
			for(int i=1;i<11;i++){
				int xplace = 10+i*stepsize;
				float y1 = graphysize-5F;
				float y2 = graphysize+5F;
				Float xfl = Float.parseFloat(xplace+"F");
				comp2D.draw(new Line2D.Float(xfl,y1,xfl,y2));
				String timstr = ""+(int)(((double)totaltime/10)*i);
				Float xfl2 = Float.parseFloat((xplace-5)+"F");
				comp2D.drawString(timstr,xfl2 , yf-10F);
			}
		}
		
		if(maxSubscriptionCost!=0){
			int stepsize = (graphysize-10)/10;
			for(int i=1;i<11;i++){
				int yplace = graphysize-i*stepsize;
				float yfl = Float.parseFloat(yplace+"F");
				float x1 = 5F;
				float x2 = 15F;
				comp2D.draw(new Line2D.Float(x1,yfl,x2,yfl));
				String subStr = ((maxSubscriptionCost*3)/10)*i+"";
				float yfl2 = Float.parseFloat((yplace+5)+"F");
				comp2D.drawString(subStr, 15F, yfl2);
			}
		}
		
		for(int i=0;i<lines.size();i++){
			GraphLine gl = lines.get(i);
			comp2D.setColor(gl.getColor());
			String namex = (graphxsize+20)+"F";
			int inty = i*15 + 20;
			String namey = inty+"F";
			comp2D.drawString(gl.getName(), Float.parseFloat(namex), Float.parseFloat(namey));
			ArrayList<Line2D.Float> lineList = gl.getLines();
			for(int j=0;j<lineList.size();j++){
				comp2D.draw(lineList.get(j));
			}
		}
		
		
	}
	

}
