/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.visual.graph;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;

/**
 * The Class ConsumerExpensePanel is used to present information 
 * about consumers expences in the market compared to the initial
 * conditions that were set.
 */
public class ConsumerExpensePanel extends GraphPanel{

	/** The Constant serialVersionUID, used for serialization */
	private static final long serialVersionUID = -2952443785206227601L;
	
	/** The highest subscription rate. */
	double highestsubrate;
	
	/** The total subscription costs. */
	double totalsubcosts;
	
	/** The providers. */
	int providers;
	
	/** A parameter used as a buffer multiplier for the y axis of the graph. */
	double yaxismultiplier = 1.5;


	/**
	 * The Constructor of the panel.
	 */
	public ConsumerExpensePanel(){
		this.setBackground(background);
		lines = new ArrayList<GraphLine>();
		time = 0;
		totaltime = 0;
		highestsubrate = 0;
		totalsubcosts =0;
		providers = 0;
		lines.add(new GraphLine("Average consumer expense in market",Color.red, 10, graphysize));
		lines.add(new GraphLine("Average initial subscription cost (all providers)",Color.blue,10,graphysize));
	}


	/**
	 * Adds a new provider
	 * 
	 * @param name
	 *            the name of the provider
	 * @param subcost
	 *            the subscription costs of this provider
	 */
	public void addProvider(int subcost, String name){
		if(subcost>highestsubrate)highestsubrate=subcost;
		totalsubcosts+=subcost;
		providers++;
	}

	/**
	 * Adds a new point to the graph
	 * 
	 * @param time
	 *            the time of the point
	 * @param avgcost
	 *            the average costs payed by the consumers
	 */
	public void addPoint(double avgcost, int time){
		if(this.time!=time){this.time=time;}
		int x = 10 + (int)(time*timestep);
		double yaxmax = highestsubrate*yaxismultiplier;
		double pct = (avgcost/yaxmax);
		int y = graphysize-(int)(pct*(graphysize-10)); 
		lines.get(0).addPoint(x, y);
		double avgsubcost = (double)totalsubcosts/providers;
		double pct2 = (avgsubcost/yaxmax);
		int y2 = graphysize-(int)(pct2*(graphysize-10));
		lines.get(1).addPoint(x, y2);
	}





	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics comp){
		super.paintComponent(comp); 
		Graphics2D comp2D = (Graphics2D) comp; 
		comp2D.setColor(background); 
		comp2D.fillRect(0, 0, xsize, ysize); 
		comp2D.setColor(Color.black);
		float yf = Float.parseFloat(graphysize+"F");
		float xf = Float.parseFloat(graphxsize+"F");
		comp2D.draw(new Line2D.Float(10F,yf,xf+10F,yf));
		comp2D.draw(new Line2D.Float(10F,yf,10F,10F));
		comp2D.drawString("Average cost per consumer", Float.parseFloat(((xsize/2)-30)+"F"), 20F);
		comp2D.drawString("TIME: "+time, Float.parseFloat(((xsize/2)-30)+"F"), 35F);
		if(totaltime!=0){
			int stepsize = (int)(((double)totaltime/10)*timestep);
			for(int i=1;i<11;i++){
				int xplace = 10+i*stepsize;
				float y1 = yf-5F;
				float y2 = yf+5F;
				Float xfl = Float.parseFloat(xplace+"F");
				comp2D.draw(new Line2D.Float(xfl,y1,xfl,y2));
				String timstr = ""+(int)(((double)totaltime/10)*i);
				Float xfl2 = Float.parseFloat((xplace-5)+"F");
				comp2D.drawString(timstr,xfl2 , yf-10F);
			}
			int stepsizey = (graphysize-10)/10;
			for(int i=1;i<11;i++){
				int yplace = graphysize-i*stepsizey;
				float yfl = Float.parseFloat(yplace+"F");
				float x1 = 5F;
				float x2 = 15F;
				comp2D.draw(new Line2D.Float(x1,yfl,x2,yfl));
				String subStr = (int)((highestsubrate*yaxismultiplier)/10)*i+"";
				float yfl2 = Float.parseFloat((yplace+5)+"F");
				comp2D.drawString(subStr, 15F, yfl2);
			}
		}
		for(int i=0;i<lines.size();i++){
			GraphLine gl = lines.get(i);
			comp2D.setColor(gl.getColor());
			String namex = 40+"F";
			int inty = i*15 + 50;
			String namey = inty+"F";
			comp2D.drawString(gl.getName(), Float.parseFloat(namex), Float.parseFloat(namey));
			ArrayList<Line2D.Float> lineList = gl.getLines();
			for(int j=0;j<lineList.size();j++){
				comp2D.draw(lineList.get(j));
			}
		}

	}
}
