/*
 * EcoSim is developed by Martin Brke as a master thesis at NTNU
 * in cooperation with Accenture.
 * 
 * For more information, contact Martin Brke at martinb@stud.ntnu.no
 */
package edu.ntnu.ecosim.agents.serviceproviders;

import edu.ntnu.ecosim.agents.serviceproviders.innovation.Innovation;
import edu.ntnu.ecosim.agents.serviceproviders.innovation.NetEffInnovation;
import edu.ntnu.ecosim.agents.serviceproviders.innovation.ValueInnovation;
import edu.ntnu.ecosim.basicfunctionality.Config;
import edu.ntnu.ecosim.exceptions.PropertyNotFoundException;
import java.util.Random;

/**
 * The Class InnovationLab represents the innovation producing part of a service provider. The development process is consisting of a chaotic exploration stage, followed by a sequential exploitation stage. 
 * @author  martin.andreas.borke
 */
public class InnovationLab {

	/** The innovation strength. */
	private double innovationStrength;
	
	/** The innovation type. */
	private int innovationType;
	
	/** The random numbers generator of this class */
	private Random rand;
	
	/** The highest value that may be added (when a value adding innovation is developed). */
	private double highestAdd;
	
	/** Whether it should be explored. */
	private boolean explore;
	
	/** How far into exploitation the innovative development process has come */
	private double exploitation;
	
	/** Factor used in the caotic process */
	private double x;
	
	/** Factor used in the chaotic process */
	private double chaos = 3.95;
	
	/** The innovation that is produced. */
	private Innovation inno;
	
	/**
	 * The Constructor.
	 * 
	 * @param innotype
	 *            a number between 0 and 10, where 0 is "only value adding" and 10 is "only network effect adding"
	 * @param strength
	 *            the innovative strength of the company (between 0 and 10)
	 */
	protected InnovationLab(final int strength, final int innotype){
		innovationType = innotype;
		inno = null;
		explore = true;
		exploitation = 50;
		rand = new Random();
		x = rand.nextDouble();
		try{
			final double highAddPct = Double.parseDouble(Config.getInstance().getProperty("ecosim", "Innovation.HighestValueAddPct"));
			highestAdd = highAddPct/100;
		}
		catch(final PropertyNotFoundException e){
			highestAdd = 0.1;
		}
		
		if(strength>10){
			innovationStrength = 51;
		}
		else if(strength<=0){
			innovationStrength = 5;
		} else {
			innovationStrength = (5*strength)+1;
		}
	}
	
	/**
	 * Creates the innovation.
	 * 
	 * @return the innovation
	 */
	private Innovation createInnovation(){
		final int random = rand.nextInt(11);
		if((random<=innovationType)&&innovationType!=0){
			final double f = exploitation/2;
			final double neff = rand.nextDouble()*(f/10);
			return new NetEffInnovation(neff);
		}
		else{
			final double valueadd = rand.nextDouble()*highestAdd;
			return new ValueInnovation(valueadd);
		}
	}
	
	
	/**
	 * Method used when development is in exoploitation stage
	 */
	private void exploite() {
		if(exploitation<2){
			inno = createInnovation();
			x = rand.nextDouble();
			explore = true;
			exploitation = 50;
		}
		else{
			final double factor = ((double)innovationStrength/5) + rand.nextDouble();
			final double newex = exploitation/factor;
			exploitation = (int)Math.ceil(newex);
		}
	}

	/**
	 * Method used when development is in exploration stage.
	 */
	private void explore(){
		x = chaos*x*(1-x);
		if(x<(innovationStrength/100)){
			explore = false;
		}
	}
	
	/**
	 * This method is used to proceed in the innovation process. 
	 * 
	 * @return the innovation (is "null" until a real innovaiton is produced)
	 */
	protected Innovation innovate(){
		if(inno!=null){
			inno = null;
		}
		if(explore){
			explore();
		}
		else{
			exploite();
		}
		return inno;
	}

}
